#! /usr/bin/env python

# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import numpy as np
from scipy import stats

from pesummary.command_line import command_line
from pesummary.utils.utils import logger
from pesummary.webpage import webpage
from pesummary.inputs import Input, PostProcessing
from pesummary.summaryplots import PlotGeneration
from pesummary.finish import FinishingTouches
from pesummary.file.meta_file import MetaFile

__doc__ == "Classes to generate webpages"


class WebpageGeneration(PostProcessing):
    """Class to generate all webpages

    Parameters
    ----------
    parser: argparser
        The parser containing the command line arguments

    Attributes
    ----------
    navbar_for_homepage: list
        List containing the structure of the homepage navbar for each results
        file
    navbar_for_approximant_homepage: list
        List containing the structure of the approximant homepage navbar for
        each results file
    navbar_for_comparison_homepage: list
        List containing the structure of the comparison navbar
    """
    def __init__(self, inputs, colors="default"):
        super(WebpageGeneration, self).__init__(inputs, colors)
        logger.info("Starting to generate webpages")
        self.same_parameters = []
        self.navbar_for_homepage = []
        self.navbar_for_approximant_homepage = []
        self.navbar_for_comparison_homepage = []
        self.image_path = {"home": "./plots/", "other": "../plots/"}
        self.results_path = {"home": "./samples", "other": "../samples"}
        self.generate_webpages()
        logger.info("Finished generating webpages")
        logger.debug("Tailoring the javascript")
        self.generate_specific_javascript()
        logger.debug("Finished Tailoring the javascript")

    @property
    def navbar_for_homepage(self):
        return self._navbar_for_homepage

    @navbar_for_homepage.setter
    def navbar_for_homepage(self, navbar_for_homepage):
        approximant_links = self._approximant_navbar_links()
        links = ["home", ["Approximants", approximant_links]]
        if len(self.approximant) > 1:
            links[1][1] = links[1][1] + ["Comparison"]
        self._navbar_for_homepage = links

    @property
    def navbar_for_approximant_homepage(self):
        return self._navbar_for_approximant_homepage

    @navbar_for_approximant_homepage.setter
    def navbar_for_approximant_homepage(self, navbar_for_approximant_homepage):
        links = [["1d_histograms", [{"multiple": i}]] for i in self.labels]
        for num, i in enumerate(self.parameters):
            for j in self._categorize_parameters(i):
                j = [j[0], [{k: self.labels[num]} for k in j[1]]]
                links[num].append(j)
        final_links = [[
            "home", ["Approximants", self._approximant_navbar_links()],
            {"corner": self.labels[num]}, {"config": self.labels[num]}, j]
            for num, j in enumerate(links)]
        if len(self.approximant) > 1:
            for i in final_links:
                i[1][1] = i[1][1] + ["Comparison"]
        self._navbar_for_approximant_homepage = final_links

    @property
    def navbar_for_comparison_homepage(self):
        return self._navbar_for_comparison_homepage

    @navbar_for_comparison_homepage.setter
    def navbar_for_comparison_homepage(self, navbar_for_comparison_homepage):
        links = ["1d_histograms", ["multiple"]]
        for i in self._categorize_parameters(self.same_parameters):
            links.append(i)
        final_links = [
            "home", ["Approximants", self._approximant_navbar_links()], links]
        final_links[1][1] = final_links[1][1] + ["Comparison"]
        self._navbar_for_comparison_homepage = final_links

    @property
    def comparison_statistics(self):
        data = []
        for ind, j in enumerate(self.same_parameters):
            indices = [k.index("%s" % (j)) for k in self.parameters]
            param_samples = [[k[indices[num]] for k in l] for num, l in
                             enumerate(self.samples)]
            data.append(self.compute_comparison_statistics(param_samples))
        return data

    @staticmethod
    def compute_comparison_statistics(samples):
        """Return the comparison statistics for a list of PDFs

        Parameters
        ----------
        samples: list
            list of samples for different PDFs
        """
        rows = range(len(samples))
        columns = range(len(samples))
        kernel = [stats.gaussian_kde(i) for i in samples]
        x = np.linspace(np.min([np.min(i) for i in samples]),
                        np.max([np.max(i) for i in samples]), 100)
        ks = [[WebpageGeneration._kolmogorov_smirnov_test(samples[i],
              samples[j]) for i in rows] for j in columns]
        js = [[WebpageGeneration._jenson_shannon_divergence(kernel[i](x),
              kernel[j](x)) for i in rows] for j in columns]
        return [ks, js]

    @staticmethod
    def _kolmogorov_smirnov_test(a, b):
        """Return the KS p value between two PDFs

        Parameters
        ----------
        a: list
            List containing the first PDF that you would like to compare
        b: list
            List containing the second PDF that you would like to compare
        """
        return stats.ks_2samp(a, b)[1]

    @staticmethod
    def _jenson_shannon_divergence(a, b):
        """Return the JS divergence test between two PDFs

        Parameters
        ----------
        a: list
            List containing the first PDF that you would like to compare
        b: list
            List containing the second PDF that you would like to compare
        """
        a = np.asarray(a)
        b = np.asarray(b)
        a /= a.sum()
        b /= b.sum()
        m = 1. / 2 * (a + b)
        kl_forward = stats.entropy(a, qk=m)
        kl_backward = stats.entropy(b, qk=m)
        return kl_forward / 2. + kl_backward / 2.

    def _approximant_navbar_links(self):
        """Return the navbar structure for the approximant tab. If we have
        passed a coherence test, then we need to prepend the approximant by
        a label so we can determine which tab corresponds to which network.
        """
        if self.coherence_test:
            labels = [i[len(self.gracedb) + 1:] if self.gracedb else i for i in
                      self.labels]
            links = [None] * len(self.approximant)
            duplicates = dict(set(
                (x, self.approximant.count(x)) for x in filter(
                    lambda rec: self.approximant.count(rec) > 1,
                    self.approximant)))
            if len(duplicates.keys()) > 0:
                for num, i in enumerate(self.approximant):
                    if i in duplicates.keys():
                        links[num] = "_".join([labels[num], i])
            return [{i: j} if k is None else {k: "None"} for i, j, k in zip(
                self.approximant, self.labels, links)]
        return [{i: j} for i, j in zip(self.approximant, self.labels)]

    def _categorize_parameters(self, parameters):
        """Categorize the parameters into common headings.

        Parameters
        ----------
        parameters: list
            List of parameters that you would like to sort
        """
        params = []
        if any("mass" in j for j in parameters):
            cond = self._condition(["mass", "q", "symmetric_mass_ratio"],
                                   ["source"])
            params.append(["masses", self._partition(cond, parameters)])
            cond = self._condition(["source"], [])
            params.append(["source", self._partition(cond, parameters)])
        if any("theta" in j for j in parameters):
            cond = self._condition(["theta", "iota"], [])
            params.append(["inclination", self._partition(cond, parameters)])
        if any("a_1" in j for j in parameters):
            cond = self._condition(["spin", "chi_p", "chi_eff", "a_1", "a_2"],
                                   [])
            params.append(["spins", self._partition(cond, parameters)])
        if any("phi" in j for j in parameters):
            cond = self._condition(["phi", "tilt"], [])
            params.append(["spin_angles", self._partition(cond, parameters)])
        if any("ra" in j for j in parameters):
            cond = self._condition(
                ["ra", "dec", "psi", "luminosity_distance", "redshift",
                 "comoving_distance"], ["mass_ratio"])
            params.append(["location", self._partition(cond, parameters)])
        if any("geocent_time" in j for j in parameters):
            cond = self._condition(["geocent_time"], [])
            params.append(["timings", self._partition(cond, parameters)])
        if any("snr" in j for j in parameters):
            cond = self._condition(["snr"], [])
            params.append(["SNR", self._partition(cond, parameters)])
        if any("phase" in j for j in parameters):
            cond = self._condition(["phase", "likelihood"], [])
            params.append(["others", self._partition(cond, parameters)])
        return params

    def _condition(self, true, false):
        """Set up a condition.

        Parameters
        ----------
        true: list
            list of strings that you would like to include
        false: list
            list of strings that you would like to neglect
        """
        if len(true) != 0 and len(false) == 0:
            condition = lambda j: True if any(i in j for i in true) else False
        if len(true) == 0 and len(false) != 0:
            condition = lambda j: True if any(i not in j for i in false) else \
                False
        if len(true) and len(false) != 0:
            condition = lambda j: True if any(i in j for i in true) and \
                any(i not in j for i in false) else False
        return condition

    def _partition(self, condition, array):
        """Filter the list according to a condition

        Parameters
        ----------
        condition: lambda function
            Lambda function containing the condition that you want to use to
            filter the array
        array: list
            List of parameters that you would like to filter
        """
        return list(filter(condition, array))

    def generate_webpages(self):
        """Generate all webpages that we need.
        """
        self.make_home_pages()
        self.make_1d_histogram_pages()
        self.make_corner_pages()
        self.make_config_pages()
        if len(self.approximant) > 1:
            self.make_comparison_pages()
        self.make_error_page()

    def _setup_page(self, html_page, links, label=None, title=None,
                    approximant=None, background_colour=None,
                    histogram_download=False):
        """Set up each webpage with a header and navigation bar.

        Parameters
        ----------
        html_page: str
            String containing the html page that you would like to set up
        links: list
            List containing the navbar structure that you would like to include
        label: str, optional
            The label that prepends your webpage name
        title: str, optional
            String that you would like to include in your header
        approximant: str, optional
            The approximant that you would like associated with your html_page
        background_colour: str, optional
            String containing the background colour of your header
        histogram_download: bool, optional
            If true, a download link for the each histogram is displayed in
            the navbar
        """
        html_file = webpage.open_html(
            web_dir=self.webdir, base_url=self.baseurl, html_page=html_page,
            label=label)
        html_file.make_header(
            approximant=approximant)
        if html_page == "home" or html_page == "home.html":
            html_file.make_navbar(
                links=links, samples_path=self.results_path["home"],
                background_color=background_colour,
                hdf5=self.hdf5)
        elif histogram_download:
            html_file.make_navbar(
                links=links, samples_path=self.results_path["home"],
                histogram_download="../samples/dat/%s_%s/%s_%s_samples.dat" % (
                    label, approximant, label, html_page),
                background_color=background_colour,
                hdf5=self.hdf5)
        else:
            html_file.make_navbar(
                links=links, samples_path=self.results_path["other"],
                background_color=background_colour,
                hdf5=self.hdf5)
        return html_file

    def make_home_pages(self):
        """Make the home pages.
        """
        pages = ["%s_%s" % (self.labels[num], i) for num, i in enumerate(
            self.approximant)]
        pages.append("home")
        webpage.make_html(web_dir=self.webdir, pages=pages)
        if self.gracedb:
            html_file = self._setup_page(
                "home", self.navbar_for_homepage,
                title="Parameter Estimation Summary Pages for %s" % (
                    self.gracedb))
        else:
            html_file = self._setup_page("home", self.navbar_for_homepage)
        path = self.image_path["home"]
        image_contents = []
        if self.psds:
            image_contents.append(path + "psd_plot.png")
        if len(self.approximant) > 1:
            image_contents.append(path + "compare_time_domain_waveforms.png")
        else:
            waveform_plot = "%s_%s_waveform_timedomain.png" % (
                self.labels[0], self.approximant[0])
            image_contents.append(path + waveform_plot)
        if self.calibration:
            image_contents.append(path + "calibration_plot.png")
        image_contents = [image_contents]
        html_file.make_table_of_images(contents=image_contents)
        images = [y for x in image_contents for y in x]
        html_file.make_modal_carousel(images=images)
        html_file.close()
        key_data = self._key_data()
        table_data = [{j: i[j] for j in self.same_parameters} for i in
                      key_data]
        contents = []
        for i in self.same_parameters:
            row = []
            row.append(i)
            for j in range(len(table_data)):
                row.append(np.round(table_data[j][i]["maxL"], 3))
            for j in range(len(table_data)):
                row.append(np.round(table_data[j][i]["mean"], 3))
            for j in range(len(table_data)):
                row.append(np.round(table_data[j][i]["median"], 3))
            for j in range(len(table_data)):
                row.append(np.round(table_data[j][i]["std"], 3))
            contents.append(row)
        for num, i in enumerate(self.approximant):
            html_file = self._setup_page(
                i, self.navbar_for_approximant_homepage[num],
                self.labels[num], title="%s Summary page" % (i),
                background_colour=self.colors[num], approximant=i)
            path = self.image_path["other"]
            label = self.labels[num]
            image_contents = [[path + "%s_1d_posterior_%s_mass_1.png" % (label, i),
                               path + "%s_1d_posterior_%s_mass_2.png" % (label, i),
                               path + "%s_1d_posterior_%s_a_1.png" % (label, i),
                               path + "%s_1d_posterior_%s_a_2.png" % (label, i)],
                              [path + "%s_%s_skymap.png" % (label, i),
                               path + "%s_%s_waveform.png" % (label, i),
                               path + "%s_1d_posterior_%s_iota.png" % (label, i),
                               path + "%s_1d_posterior_%s_luminosity_distance.png" % (label, i)]]
            html_file.make_table_of_images(contents=image_contents)
            images = [y for x in image_contents for y in x]
            html_file.make_modal_carousel(images=images)
            table_contents = []
            for i in contents:
                one_approx_content = [i[0]] + [i[j * len(self.approximant)
                                               + num + 1] for j in range(4)]
                table_contents.append(one_approx_content)
            html_file.make_table(headings=[" ", "maxL", "mean", "median", "std"],
                                 contents=table_contents, heading_span=1)
            html_file.make_footer(user=self.user, rundir=self.webdir)
            html_file.close()

    def make_1d_histogram_pages(self):
        """Make the 1d histogram pages.
        """
        pages = ["%s_%s_%s" % (self.labels[num], i, j) for num, i in
                 enumerate(self.approximant) for j in self.parameters[num]]
        pages += ["%s_%s_multiple" % (self.labels[num], i) for num, i in
                  enumerate(self.approximant)]
        webpage.make_html(web_dir=self.webdir, pages=pages)
        for num, app in enumerate(self.approximant):
            for j in self.parameters[num]:
                html_file = self._setup_page(
                    "%s_%s" % (app, j),
                    self.navbar_for_approximant_homepage[num],
                    self.labels[num], title="%s Posterior PDF for %s" % (
                        app, j),
                    approximant=app, background_colour=self.colors[num],
                    histogram_download=True)
                path = self.image_path["other"]
                label = self.labels[num]
                contents = [[path + "%s_1d_posterior_%s_%s.png" % (label, app, j)],
                            [path + "%s_sample_evolution_%s_%s.png" % (label, app, j),
                             path + "%s_autocorrelation_%s_%s.png" % (label, app, j)]]
                html_file.make_table_of_images(
                    contents=contents, rows=1, columns=2, code="changeimage")
                html_file.make_footer(user=self.user, rundir=self.webdir)
                html_file.close()
            html_file = self._setup_page(
                "%s_multiple" % (app),
                self.navbar_for_approximant_homepage[num],
                self.labels[num], title="%s Posteriors for multiple" % (app),
                approximant=app, background_colour=self.colors[num])
            ordered_parameters = self._categorize_parameters(self.parameters[num])
            ordered_parameters = [i for j in ordered_parameters for i in j[1]]
            html_file.make_search_bar(sidebar=[i for i in self.parameters[num]],
                                      popular_options=[
                                          "mass_1, mass_2",
                                          "luminosity_distance, iota, ra, dec",
                                          "iota, phi_12, phi_jl, tilt_1, tilt_2",
                                          {"all": ", ".join(ordered_parameters)}],
                                      label=self.labels[num],
                                      code="combines")
            html_file.make_footer(user=self.user, rundir=self.webdir)
            html_file.close()

    def make_corner_pages(self):
        """Make the corner pages.
        """
        pages = ["%s_%s_corner" % (self.labels[num], i) for num, i in
                 enumerate(self.approximant)]
        webpage.make_html(web_dir=self.webdir, pages=pages)
        for num, app in enumerate(self.approximant):
            html_file = self._setup_page(
                "%s_corner" % (app),
                self.navbar_for_approximant_homepage[num],
                self.labels[num], title="%s Corner Plots" % (app),
                background_colour=self.colors[num], approximant=app)
            params = ["luminosity_distance", "dec", "a_2", "phase",
                      "a_1", "geocent_time", "phi_jl", "psi", "ra",
                      "mass_2", "mass_1", "phi_12", "tilt_2", "iota",
                      "tilt_1", "chi_p", "chirp_mass", "mass_ratio",
                      "symmetric_mass_ratio", "total_mass", "chi_eff",
                      "redshift", "mass_1_source", "mass_2_source",
                      "total_mass_source", "chirp_mass_source"]
            included_parameters = [i for i in self.parameters[num] if i in params]
            html_file.make_search_bar(sidebar=included_parameters,
                                      popular_options=[
                                          "mass_1, mass_2",
                                          "luminosity_distance, iota, ra, dec",
                                          "iota, phi_12, phi_jl, tilt_1, tilt_2"],
                                      label=self.labels[num])
            html_file.make_footer(user=self.user, rundir=self.webdir)
            html_file.close()

    def make_config_pages(self):
        """Make the configuration pages.
        """
        pages = ["%s_%s_config" % (self.labels[num], i) for num, i in
                 enumerate(self.approximant)]
        webpage.make_html(web_dir=self.webdir, pages=pages, stylesheets=pages)
        for num, app in enumerate(self.approximant):
            html_file = self._setup_page(
                "%s_config" % (app),
                self.navbar_for_approximant_homepage[num],
                self.labels[num], title="%s configuration" % (app),
                background_colour=self.colors[num], approximant=app)
            if self.config and num < len(self.config):
                with open(self.config[num], 'r') as f:
                    contents = f.read()
                styles = html_file.make_code_block(language='ini', contents=contents)
                with open('{0:s}/css/{1:s}_{2:s}_config.css'.format(self.webdir,
                          self.labels[num], app), 'w') as f:
                    f.write(styles)
            else:
                html_file.add_content(
                    "<div class='row justify-content-center'>"
                    "<p style='margin-top:2.5em'> No configuration file was "
                    "provided </p></div>")
            html_file.make_footer(user=self.user, rundir=self.webdir)
            html_file.close()

    def make_comparison_pages(self):
        """Make the comparison pages.
        """
        webpage.make_html(web_dir=self.webdir, pages=["Comparison"])
        html_file = self._setup_page(
            "Comparison", self.navbar_for_comparison_homepage,
            title="Comparison Summary Page", approximant="Comparison")
        path = self.image_path["other"]
        contents = [[path + "combined_skymap.png", path + "compare_waveforms.png"]]
        html_file.make_table_of_images(contents=contents)
        images = [y for x in contents for y in x]
        html_file.make_modal_carousel(images=images)
        if self.sensitivity:
            html_file.add_content(
                "<div class='row justify-content-center' "
                "style='margin=top: 2.0em;'><p>To see the sky sensitivity for "
                "the following networks, click the button</p></div>")
            html_file.add_content(
                "<div class='row justify-content-center' "
                "style='margin-top: 0.2em;'><button type='button' class='btn "
                "btn-info' onclick='%s.src=\"%s/plots/combined_skymap.png\"'"
                "style='margin-left:0.25em; margin-right:0.25em'>Sky Map</button>"
                "<button type='button' class='btn btn-info' onclick='%s.src=\""
                "%s/plots/%s_sky_sensitivity_HL.png\"'"
                "style='margin-left:0.25em; margin-right:0.25em'>HL</button>"
                % ("combined_skymap", self.baseurl, "combined_skymap", self.baseurl,
                   self.approximant[0]))
            html_file.add_content(
                "<button type='button' class='btn btn-info' "
                "onclick='%s.src=\"%s/plots/%s_sky_sensitivity_HLV.png\"'"
                "style='margin-left:0.25em; margin-right:0.25em'>HLV</button></div>\n"
                % ("combined_skymap", self.baseurl, self.approximant[0]))
        try:
            statistics = self.comparison_statistics
        except Exception as e:
            statistics = None
            logger.info("Failed to generate comparison statistics because "
                        "%s" % (e))
        if statistics:
            rows = range(len(self.approximant))
            labels = ["_".join([i, j]) if i is not None else j for i, j in zip(
                self.label_to_prepend_approximant, self.approximant)]
            style_ks = "margin-top:5em; margin-bottom:1em; background-color:#FFFFFF; " + \
                "box-shadow: 0 0 5px grey;"
            style_js = "margin-top:0em; margin-bottom:5em; background-color:#FFFFFF; " + \
                "box-shadow: 0 0 5px grey;"
            table_contents = {self.same_parameters[num]: [
                [labels[i]] + statistics[num][0][i] for i in rows]
                for num in range(len(self.same_parameters))}
            html_file.make_table(headings=[" "] + labels,
                                 contents=table_contents, heading_span=1,
                                 accordian_header="KS test total", style=style_ks)
            table_contents = {self.same_parameters[num]: [
                [labels[i]] + statistics[num][1][i] for i in rows]
                for num in range(len(self.same_parameters))}
            html_file.make_table(headings=[" "] + labels,
                                 contents=table_contents, heading_span=1,
                                 accordian_header="JS divergence test total", style=style_js)
        html_file.make_footer(user=self.user, rundir=self.webdir)
        pages = ["Comparison_%s" % (i) for i in self.same_parameters]
        pages += ["Comparison_multiple"]
        webpage.make_html(web_dir=self.webdir, pages=pages)
        for num, i in enumerate(self.same_parameters):
            html_file = self._setup_page(
                "Comparison_%s" % (i),
                self.navbar_for_comparison_homepage,
                title="Comparison PDF for %s" % (i),
                approximant="Comparison")
            html_file.insert_image(path + "combined_1d_posterior_%s.png" % (i),
                                   code="changeimage")
            if statistics:
                table_contents = [
                    [labels[i]] + statistics[num][0][i] for i in rows]
                html_file.make_table(headings=[" "] + labels,
                                     contents=table_contents, heading_span=1,
                                     accordian_header="KS test", style=style_ks)
                table_contents = [
                    [labels[i]] + statistics[num][1][i] for i in rows]
                html_file.make_table(headings=[" "] + labels,
                                     contents=table_contents, heading_span=1,
                                     accordian_header="JS divergence test",
                                     style=style_js)
            html_file.make_footer(user=self.user, rundir=self.webdir)
            html_file.close()
        html_file = self._setup_page(
            "Comparison_multiple", self.navbar_for_comparison_homepage,
            approximant="Comparison", title="Comparison Posteriors for multiple")
        html_file.make_search_bar(sidebar=self.same_parameters,
                                  popular_options=[
                                      "mass_1, mass_2",
                                      "luminosity_distance, iota, ra, dec",
                                      "iota, phi_12, phi_jl, tilt_1, tilt_2",
                                      {"all": ", ".join(self.same_parameters)}],
                                  label="None", code="combines")
        html_file.make_footer(user=self.user, rundir=self.webdir)

    def make_error_page(self):
        """Make a page that is shown when something goes wrong.
        """
        pages = ["error"]
        webpage.make_html(web_dir=self.webdir, pages=pages)
        html_file = webpage.open_html(
            web_dir=self.webdir, base_url=self.baseurl, html_page="error")
        html_file.make_div(_class='jumbotron text-center',
                           _style='background-color: #D3D3D3; margin-botton:0')
        html_file.make_div(_class='container',
                           _style=('margin-top:1em; background-color: #D3D3D3;'
                                   'width: 45em'))
        html_file.add_content(
            "<h1 style='color:white; font-size: 8em'>404</h1>")
        html_file.add_content(
            "<h2 style='color:white;'> Something went wrong... </h2>")
        html_file.end_div()
        html_file.end_div()
        html_file.close()

    def generate_specific_javascript(self):
        """Tailor the javascript to the specific situation.
        """
        path = self.webdir + "/js/grab.js"
        existing = open(path)
        existing = existing.readlines()
        ind = existing.index("    if ( param == approximant ) {\n")
        content = existing[:ind]
        for i in [list(j.keys())[0] for j in self._approximant_navbar_links()]:
            content.append("    if ( param == \"%s\" ) {\n" % (i))
            content.append("        approx = \"None\" \n")
            content.append("    }\n")
        for i in existing[ind + 1:]:
            content.append(i)
        new_file = open(path, "w")
        new_file.writelines(content)
        new_file.close()


def main():
    """Top level interface for `summarypages`
    """
    parser = command_line()
    opts = parser.parse_args()
    inputs = Input(opts)
    PlotGeneration(inputs)
    WebpageGeneration(inputs)
    MetaFile(inputs)
    FinishingTouches(inputs)


if __name__ == '__main__':
    main()



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import numpy as np
import h5py
from scipy import interpolate

from pesummary.file.standard_names import standard_names


class LALInferenceResultsFile(object):
    """Class to handle LALInference results format

    Attributes
    ----------
    path: str
        the path to the posterior samples stored in the hdf5 file
    """
    def __init__(self, f):
        self.fil = f
        self.path = None

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, path):
        self._data_structure = []
        f = h5py.File(self.fil)
        f.visit(self._add_to_list)
        for i in self._data_structure:
            condition1 = "posterior_samples" in i or "posterior" in i
            condition2 = "posterior_samples/" not in i and "posterior/" not in i
            if condition1 and condition2:
                self._path = i
        f.close()

    def _add_to_list(self, item):
        self._data_structure.append(item)

    def _lalinference_parameters_and_samples(self):
        """Grab the parameters and samples stored in file
        """
        samples = []
        f = h5py.File(self.fil, "r")
        lalinference_names = f[self.path].dtype.names
        parameters = [i for i in lalinference_names]
        for i in f[self.path]:
            samples.append(
                [i[lalinference_names.index(j)] for j in lalinference_names])
        return parameters, samples

    def grab_samples(self):
        """Return the posterior samples stored in the results file
        """
        samples = []
        f = h5py.File(self.fil, "r")
        lalinference_names = f[self.path].dtype.names
        parameters = [
            i for i in lalinference_names if i in standard_names.keys()]
        for i in f[self.path]:
            samples.append(
                [i[lalinference_names.index(j)] for j in parameters])
        parameters = [standard_names[i] for i in parameters]

        condition1 = "luminosity" not in parameters
        condition2 = "logdistance" in lalinference_names
        if condition1 and condition2:
            parameters.append("luminosity_distance")
            for num, i in enumerate(f[self.path]):
                samples[num].append(
                    np.exp(i[lalinference_names.index("logdistance")]))
        if "theta_jn" not in parameters and "costheta_jn" in lalinference_names:
            parameters.append("theta_jn")
            for num, i in enumerate(f[self.path]):
                samples[num].append(
                    np.arccos(i[lalinference_names.index("costheta_jn")]))
        f.close()
        return parameters, samples

    def grab_calibration_data(self):
        """Return the calibration data stored in the results file
        """
        parameters, samples = self._lalinference_parameters_and_samples()
        f = h5py.File(self.fil)
        attributes = f[self.path].attrs.items()

        log_frequencies = {
            key.split("_")[0]: [] for key, value in attributes if
            "_spcal_logfreq" in key}
        for key, value in attributes:
            if "_spcal_logfreq" in key:
                log_frequencies[key.split("_")[0]].append(float(value))

        keys_amp = np.sort([
            param for param in parameters if "_spcal_amp" in param])
        keys_phase = np.sort([
            param for param in parameters if "_spcal_phase" in
            param])

        amp_params = {ifo: [] for ifo in log_frequencies.keys()}
        phase_params = {ifo: [] for ifo in log_frequencies.keys()}
        for key in keys_amp:
            ifo = key.split("_")[0]
            ind = parameters.index(key)
            amp_params[ifo].append([float(i[ind]) for i in samples])
        for key in keys_phase:
            ifo = key.split("_")[0]
            ind = parameters.index(key)
            phase_params[ifo].append([float(i[ind]) for i in samples])
        f.close()

        total = []
        for key in log_frequencies.keys():
            f = np.exp(log_frequencies[key])
            fs = np.linspace(np.min(f), np.max(f), 100)
            data = [interpolate.spline(log_frequencies[key], samp, np.log(fs)) for samp
                    in np.column_stack(amp_params[key])]
            amplitude_upper = 1. - np.percentile(data, 90, axis=0)
            amplitude_lower = 1. - np.percentile(data, 10, axis=0)
            amplitude_median = 1. - np.median(data, axis=0)

            data = [interpolate.spline(log_frequencies[key], samp, np.log(fs)) for samp
                    in np.column_stack(phase_params[key])]

            phase_upper = np.percentile(data, 90, axis=0)
            phase_lower = np.percentile(data, 10, axis=0)
            phase_median = np.median(data, axis=0)
            total.append(np.column_stack(
                [fs, amplitude_median, phase_median, amplitude_lower,
                 phase_lower, amplitude_upper, phase_upper]))
        return total, list(log_frequencies.keys())



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os

import numpy as np
import h5py
import json
import configparser

from pesummary.utils.utils import logger
from pesummary.inputs import PostProcessing
from pesummary.file.existing import ExistingFile
from pesummary.utils.utils import make_dir


def _recursively_save_dictionary_to_hdf5_file(f, dictionary, current_path=None):
    """Recursively save a dictionary to a hdf5 file

    Parameters
    ----------
    f: h5py._hl.files.File
        the open hdf5 file that you would like the data to be saved to
    dictionary: dict
        dictionary of data
    current_path: optional, str
        string to indicate the level to save the data in the hdf5 file
    """
    try:
        f.create_group("posterior_samples")
        if "psds" in dictionary.keys():
            f.create_group("psds")
        if "calibration_envelope" in dictionary.keys():
            f.create_group("calibration_envelope")
        if "config_file" in dictionary.keys():
            f.create_group("config_file")
    except Exception:
        pass
    if current_path is None:
        current_path = []

    for k, v in dictionary.items():
        if isinstance(v, dict):
            try:
                f["/".join(current_path)].create_group(k)
            except Exception:
                pass
            path = current_path + [k]
            _recursively_save_dictionary_to_hdf5_file(f, v, path)
        elif isinstance(v, list):
            if isinstance(v[0], str):
                f["/".join(current_path)].create_dataset(k, data=np.array(
                    v, dtype="S"
                ))
            elif isinstance(v[0], list):
                f["/".join(current_path)].create_dataset(k, data=np.array(v))
        elif isinstance(v, (str, bytes)):
            f["/".join(current_path)].create_dataset(k, data=np.array(
                [v], dtype="S"
            ))


class MetaFile(PostProcessing):
    """This class handles the creation of a meta file storing all information
    from the analysis

    Attributes
    ----------
    meta_file: str
        name of the meta file storing all information
    """
    def __init__(self, inputs):
        super(MetaFile, self).__init__(inputs)
        logger.info("Starting to generate the meta file")
        self.data = {}
        self.existing_label = None
        self.existing_approximant = None
        self.existing_parameters = None
        self.existing_samples = None
        self.generate_meta_file_data()

        if not self.hdf5:
            self.save_to_json()
        else:
            self.save_to_hdf5()

        self.generate_dat_file()
        logger.info("Finished generating the meta file. The meta file can be "
                    "viewed here: %s" % (self.meta_file))

    @property
    def meta_file(self):
        if not self.hdf5:
            return self.webdir + "/samples/posterior_samples.json"
        return self.webdir + "/samples/posterior_samples.h5"

    @staticmethod
    def convert_to_list(data):
        for num, i in enumerate(data):
            if isinstance(data[num], np.ndarray):
                data[num] = list(i)
            for idx, j in enumerate(i):
                if isinstance(data[num][idx], np.ndarray):
                    data[num][idx] = list(i)
        return data

    def generate_meta_file_data(self):
        """Generate dictionary of data which will go into the meta_file
        """
        if self.existing:
            existing_file = ExistingFile(self.existing)
            self.existing_parameters = existing_file.existing_parameters
            self.existing_samples = existing_file.existing_samples
            self.existing_approximant = existing_file.existing_approximant
            self.existing_label = existing_file.existing_labels
        self._make_dictionary()

    def _make_dictionary(self):
        if self.existing_label:
            self._make_dictionary_structure(
                self.existing_label, self.existing_approximant
            )
            for num, i in enumerate(self.existing_label):
                self._add_data(i, self.existing_approximant[num],
                               self.existing_parameters[num],
                               self.existing_samples[num],
                               )
        self._make_dictionary_structure(self.labels, self.approximant,
                                        psd=self.psds,
                                        calibration=self.calibration,
                                        config=self.config
                                        )
        for num, i in enumerate(self.labels):
            psd = self._grab_psd_data_from_data_files(
                self.psds, self.psd_labels) if self.psds else None
            calibration = self._grab_calibration_data_from_data_files(
                self.calibration, self.calibration_labels) if self.calibration \
                else None
            config = self._grab_config_data_from_data_file(self.config[num]) if \
                self.config and num < len(self.config) else None
            self._add_data(i, self.approximant[num], self.parameters[num],
                           self.samples[num], psd=psd, calibration=calibration,
                           config=config
                           )

    def _grab_psd_data_from_data_files(self, files, psd_labels):
        """Return the psd data as a dictionary

        Parameters
        ----------
        files: list
            list of psd files
        psd_labels: list
            list of labels associated with each psd file
        """
        return {psd_labels[num]: [[i, j] for i, j in zip(
            self._grab_frequencies_from_psd_data_file(f),
            self._grab_strains_from_psd_data_file(f)
        )] for num, f in enumerate(files)}

    def _grab_calibration_data_from_data_files(self, files, calibration_labels):
        """Return the calibration envelope data as a dictionary

        Parameters
        ----------
        files: list
            list of calibration envelope files
        calibration_labels:
            list of labels associated with each calibration envelope file
        """
        dictionary = {}
        for num, i in enumerate(calibration_labels):
            dictionary[i] = [
                [k[0], k[1], k[2], k[3], k[4], k[5], k[6]] for k in files[num]]
        return dictionary

    def _grab_config_data_from_data_file(self, file):
        """Return the config data as a dictionary

        Parameters
        ----------
        file: str
            path to the configuration file
        """
        data = {}
        config = configparser.ConfigParser()
        try:
            config.read(file)
            sections = config.sections()
        except Exception as e:
            sections = None
            logger.info("Unable to open %s because %s. The data will not be "
                        "stored in the meta file" % (file, e))
        if sections:
            for i in sections:
                data[i] = {}
                for key in config["%s" % (i)]:
                    data[i][key] = config["%s" % (i)]["%s" % (key)]
        return data

    def _add_data(self, label, approximant, parameters, samples,
                  psd=None, calibration=None, config=None):
        """Add data to the stored dictionary

        Parameters
        ----------
        label: str
            the name of the second level in the dictionary
        approximant: str
            the name of the third level in the dictionary
        parameters: list
            list of parameters that were used in the study
        samples: list
            list of samples that wee used in th study
        psd: list, optional
            list of frequencies and strains associated with the psd
        calibration: list, optional
            list of data associated with the calibration envelope
        config: dict, optional
            data associated with the configuration file
        """
        self.data["posterior_samples"][label][approximant] = {
            "parameter_names": list(parameters),
            "samples": self.convert_to_list(samples)
        }
        if psd:
            for i in list(psd.keys()):
                self.data["psds"][label][approximant][i] = psd[i]
        if calibration:
            for i in list(calibration.keys()):
                self.data["calibration_envelope"][label][approximant][i] = \
                    calibration[i]
        if config:
            self.data["config_file"][label][approximant] = config

    def _make_dictionary_structure(self, label, approximant, psd=None,
                                   calibration=None, config=None):
        for num, i in enumerate(label):
            self._add_label_and_approximant(
                "posterior_samples", i, approximant[num]
            )
            if psd:
                self._add_label_and_approximant("psds", i, approximant[num])
            if calibration:
                self._add_label_and_approximant(
                    "calibration_envelope", i, approximant[num]
                )
            if config:
                self._add_label_and_approximant(
                    "config_file", i, approximant[num]
                )

    def _add_label_and_approximant(self, base_level, label, approximant):
        if base_level not in list(self.data.keys()):
            self.data[base_level] = {}
        if label not in list(self.data[base_level].keys()):
            self.data[base_level][label] = {}
        self.data[base_level][label][approximant] = {}

    def save_to_json(self):
        with open(self.meta_file, "w") as f:
            json.dump(self.data, f, indent=4, sort_keys=True)

    def save_to_hdf5(self):
        with h5py.File(self.meta_file, "w") as f:
            _recursively_save_dictionary_to_hdf5_file(f, self.data)

    def generate_dat_file(self):
        """Generate a single .dat file that contains all the samples for a
        given analysis
        """
        self.savedir = "%s/samples/dat" % (self.webdir)
        if not os.path.isdir(self.savedir):
            make_dir(self.savedir)
        for num, i in enumerate(self.result_files):
            if "posterior_samples.h5" not in i:
                make_dir("%s/%s_%s" % (
                    self.savedir, self.labels[num], self.approximant[num]))
                for idx, j in enumerate(self.parameters[num]):
                    data = [j[idx] for j in self.samples[num]]
                    np.savetxt("%s/%s_%s/%s_%s_%s_samples.dat" % (
                        self.savedir, self.labels[num], self.approximant[num],
                        self.labels[num], self.approximant[num], j), data,
                        fmt="%s")



#! /usr/bin/env python

# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import matplotlib
matplotlib.use("Agg")
import matplotlib.pyplot as plt

from pesummary.plot import plot
from pesummary.file.existing import ExistingFile
from pesummary.utils.utils import logger
from pesummary.command_line import command_line
from pesummary.inputs import Input, PostProcessing

import warnings

from glob import glob
import math
import numpy as np

__doc__ == "Class to generate plots"

latex_labels = {"luminosity_distance": r"$d_{L} [Mpc]$",
                "geocent_time": r"$t_{c} [s]$",
                "dec": r"$\delta [rad]$",
                "ra": r"$\alpha [rad]$",
                "a_1": r"$a_{1}$",
                "a_2": r"$a_{2}$",
                "phi_jl": r"$\phi_{JL} [rad]$",
                "phase": r"$\phi [rad]$",
                "psi": r"$\Psi [rad]$",
                "iota": r"$\iota [rad]$",
                "tilt_1": r"$\theta_{1} [rad]$",
                "tilt_2": r"$\theta_{2} [rad]$",
                "phi_12": r"$\phi_{12} [rad]$",
                "mass_2": r"$m_{2} [M_{\odot}]$",
                "mass_1": r"$m_{1} [M_{\odot}]$",
                "total_mass": r"$M [M_{\odot}]$",
                "chirp_mass": r"$\mathcal{M} [M_{\odot}]$",
                "log_likelihood": r"$\log{\mathcal{L}}$",
                "H1_matched_filter_snr": r"$\rho^{H}_{mf}$",
                "L1_matched_filter_snr": r"$\rho^{L}_{mf}$",
                "H1_optimal_snr": r"$\rho^{H}_{opt}$",
                "L1_optimal_snr": r"$\rho^{L}_{opt}$",
                "V1_optimal_snr": r"$\rho^{V}_{opt}$",
                "E1_optimal_snr": r"$\rho^{E}_{opts}$",
                "spin_1x": r"$S_{1}x$",
                "spin_1y": r"$S_{1}y$",
                "spin_1z": r"$S_{1}z$",
                "spin_2x": r"$S_{2}x$",
                "spin_2y": r"$S_{2}y$",
                "spin_2z": r"$S_{2}z$",
                "chi_p": r"$\chi_{p}$",
                "chi_eff": r"$\chi_{eff}$",
                "mass_ratio": r"$q$",
                "symmetric_mass_ratio": r"$\eta$",
                "phi_1": r"$\phi_{1} [rad]$",
                "phi_2": r"$\phi_{2} [rad]$",
                "cos_tilt_1": r"$\cos{\theta_{1}}$",
                "cos_tilt_2": r"$\cos{\theta_{2}}$",
                "redshift": r"$z$",
                "comoving_distance": r"$d_{com} [Mpc]$",
                "mass_1_source": r"$m_{1}^{source} [M_{\odot}]$",
                "mass_2_source": r"$m_{2}^{source} [M_{\odot}]$",
                "chirp_mass_source": r"$\mathcal{M}^{source} [M_{\odot}]$",
                "total_mass_source": r"$M^{source} [M_{\odot}]$",
                "cos_iota": r"$\cos{\iota}$",
                "theta_jn": r"$\theta_{JN} [rad]$"}


class PlotGeneration(PostProcessing):
    """Class to generate all available plots for each results file.

    Parameters
    ----------
    parser: argparser
        The parser containing the command line arguments

    Attributes
    ----------
    savedir: str
        The path to the directory where all plots will be saved
    """
    def __init__(self, inputs, colors="default"):
        super(PlotGeneration, self).__init__(inputs, colors)
        self.inputs = inputs
        logger.info("Starting to generate plots")
        self.generate_plots()
        logger.info("Finished generating the plots")

    @staticmethod
    def _check_latex_labels(parameters):
        for i in parameters:
            if i not in list(latex_labels.keys()):
                latex_labels[i] = i

    @property
    def savedir(self):
        return self.webdir + "/plots/"

    def generate_plots(self):
        """Generate all plots for all results files.
        """
        logger.debug("Generating the calibration plot")
        self.try_to_make_a_plot("calibration")
        logger.debug("Generating the psd plot")
        self.try_to_make_a_plot("psd")
        for num, i in enumerate(self.approximant):
            logger.debug("Starting to generate plots for %s\n" % (i))
            self._check_latex_labels(self.parameters[num])
            self.try_to_make_a_plot("corner", num)
            self.try_to_make_a_plot("skymap", num)
            self.try_to_make_a_plot("waveform", num)
            self.try_to_make_a_plot("1d_histogram", num)
        if self.sensitivity:
            self.try_to_make_a_plot("sensitivity", 0)
        if self.add_to_existing:
            existing = ExistingFile(self.existing)
            existing_config = glob(self.existing + "/config/*")
            for num, i in enumerate(existing.existing_approximant):
                original_label = existing.existing_labels[num]
                self.labels.append(original_label)
                self.approximant.append(existing.existing_approximant[num])
                self.result_files.append(existing.existing_file)
                self.samples.append(existing.existing_samples[num])
                self.parameters.append(existing.existing_parameters[num])
                if self.config and len(existing_config) > 1:
                    self.config.append(existing_config[num])
            key_data = self._key_data()
            maxL_list = []
            for idx, j in enumerate(self.parameters):
                dictionary = {k: key_data[idx][k]["maxL"] for k in j}
                dictionary["approximant"] = self.approximant[idx]
                maxL_list.append(dictionary)
            self.maxL_samples = maxL_list
            self.same_parameters = list(
                set.intersection(*[set(l) for l in self.parameters]))
        if len(self.samples) > 1:
            logger.debug("Starting to generate comparison plots\n")
            self.try_to_make_a_plot("1d_histogram_comparison", "all")
            self.try_to_make_a_plot("skymap_comparison", "all")
            self.try_to_make_a_plot("waveform_comparison", "all")

    def try_to_make_a_plot(self, plot_type, idx=None):
        """Try and make a plot. If it fails, return an error.

        Parameters
        ----------
        plot_type: str
            String for the plot that you wish to try and make
        idx: int
            The index of the results file that you wish to analyse.
        """
        plot_type_dictionary = {"calibration": self._calibration_plot,
                                "psd": self._psd_plot,
                                "corner": self._corner_plot,
                                "skymap": self._skymap_plot,
                                "waveform": self._waveform_plot,
                                "1d_histogram": self._1d_histogram_plots,
                                "1d_histogram_comparison":
                                self._1d_histogram_comparison_plots,
                                "skymap_comparison":
                                self._skymap_comparison_plot,
                                "waveform_comparison":
                                self._waveform_comparison_plot,
                                "sensitivity": self._sensitivity_plot}
        try:
            plot_type_dictionary[plot_type](idx)
        except Exception as e:
            logger.info("Failed to generate %s plot because "
                        "%s" % (plot_type, e))

    def _calibration_plot(self, idx=None):
        """Generate a single plot showing the calibration envelopes for all
        IFOs used in the analysis.
        """
        frequencies = np.arange(20., 1024., 1. / 4)
        fig = plot._calibration_envelope_plot(
            frequencies, self.calibration, self.calibration_labels)
        fig.savefig("%s/calibration_plot.png" % (self.savedir))
        plt.close()

    def _psd_plot(self, idx=None):
        """Generate a single plot showing all psds used in analysis
        """
        frequencies = [self._grab_frequencies_from_psd_data_file(i) for i in
                       self.psds]
        strains = [self._grab_strains_from_psd_data_file(i) for i in self.psds]
        fig = plot._psd_plot(frequencies, strains, labels=self.psd_labels)
        fig.savefig("%s/psd_plot.png" % (self.savedir))
        plt.close()

    def _corner_plot(self, idx):
        """Generate a corner plot for a given results file.

        Parameters
        ----------
        idx: int
            The index of the results file that you wish to analyse
        """
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            fig, params = plot._make_corner_plot(
                self.samples[idx], self.parameters[idx], latex_labels)
            plt.savefig("%s/corner/%s_%s_all_density_plots.png" % (
                self.savedir, self.labels[idx], self.approximant[idx]))
            plt.close()
            combine_corner = open("%s/js/combine_corner.js" % (self.webdir))
            combine_corner = combine_corner.readlines()
            params = [str(i) for i in params]
            for linenumber, line in enumerate(combine_corner):
                if "var list = [" in line:
                    combine_corner[linenumber] = "    var list = %s;\n" % (
                        params)
            new_file = open("%s/js/combine_corner.js" % (self.webdir), "w")
            new_file.writelines(combine_corner)
            new_file.close()

    def _skymap_plot(self, idx):
        """Generate a skymap showing the confidence regions for a given results
        file.


        Parameters
        ----------
        idx: int
            The index of the results file that you wish to analyse
        """
        ind_ra = self.parameters[idx].index("ra")
        ind_dec = self.parameters[idx].index("dec")
        ra = [j[ind_ra] for j in self.samples[idx]]
        dec = [j[ind_dec] for j in self.samples[idx]]
        fig = plot._sky_map_plot(ra, dec)
        fig.savefig(self.savedir + "/%s_%s_skymap.png" % (
            self.labels[idx], self.approximant[idx]))
        plt.close()

    def _sensitivity_plot(self, idx):
        """Generate a plot showing the network sensitivity for the maximum
        likelihood waveform.

        Parameters
        ----------
        idx: int
            The index of the results file that you wish to analyse
        """
        fig = plot._sky_sensitivity(["H1", "L1"], 0.2, self.maxL_samples[idx])
        plt.savefig(self.savedir + "%s_sky_sensitivity_HL" % (
            self.approximant[idx]))
        plt.close()
        fig = plot._sky_sensitivity(
            ["H1", "L1", "V1"], 0.2, self.maxL_samples[idx])
        fig.savefig(self.savedir + "%s_sky_sensitivity_HLV" % (
            self.approximant[idx]))
        plt.close()

    def _waveform_plot(self, idx):
        """Generate a plot showing the maximum likelihood waveform in each
        available detector.

        Parameters
        ----------
        idx: int
            The index of the results file that you wish to analyse
        """
        if self.detectors[idx] is None:
            detectors = ["H1", "L1"]
        else:
            detectors = self.detectors[idx].split("_")
        fig = plot._waveform_plot(detectors, self.maxL_samples[idx])
        plt.savefig(self.savedir + "%s_%s_waveform.png" % (
            self.labels[idx], self.approximant[idx]))
        plt.close()
        fig = plot._time_domain_waveform(detectors, self.maxL_samples[idx])
        fig.savefig(self.savedir + "%s_%s_waveform_timedomain.png" % (
            self.labels[idx], self.approximant[idx]))
        plt.close()

    def _1d_histogram_plots(self, idx):
        """Generate 1d_histogram plots, sample evolution plots, plots
        showing the autocorrelation function and the CDF plots for all
        parameters in the results file.

        Parameters
        ----------
        idx: int
            The index of the results file that you wish to analyse
        """
        for ind, j in enumerate(self.parameters[idx]):
            try:
                index = self.parameters[idx].index("%s" % (j))
                inj_value = self.injection_data[idx]["%s" % (j)]
                if math.isnan(inj_value):
                    inj_value = None
                param_samples = [k[index] for k in self.samples[idx]]
                fig = plot._1d_histogram_plot(
                    j, param_samples, latex_labels[j], inj_value)
                plt.savefig(self.savedir + "%s_1d_posterior_%s_%s.png" % (
                    self.labels[idx], self.approximant[idx], j))
                plt.close()
                fig = plot._sample_evolution_plot(
                    j, param_samples, latex_labels[j], inj_value)
                plt.savefig(self.savedir + "%s_sample_evolution_%s_%s.png" % (
                    self.labels[idx], self.approximant[idx], j))
                plt.close()
                fig = plot._autocorrelation_plot(j, param_samples)
                plt.savefig(self.savedir + "%s_autocorrelation_%s_%s.png" % (
                    self.labels[idx], self.approximant[idx], j))
                plt.close()
                fig = plot._1d_cdf_plot(j, param_samples, latex_labels[j])
                fig.savefig(self.savedir + "%s_cdf_%s_%s.png" % (
                    self.labels[idx], self.approximant[idx], j))
                plt.close()
            except Exception as e:
                logger.info("Failed to generate 1d_histogram plots for %s "
                            "because %s" % (j, e))
                continue

    def _1d_histogram_comparison_plots(self, idx="all"):
        """Generate comparison plots for all parameters that are consistent
        across all results files.

        Parameters
        ----------
        idx: int, optional
            The indicies of the results files that you wish to be included
            in the comparsion plots.
        """
        for ind, j in enumerate(self.same_parameters):
            try:
                indices = [k.index("%s" % (j)) for k in self.parameters]
                param_samples = [[k[indices[num]] for k in l] for num, l in
                                 enumerate(self.samples)]
                fig = plot._1d_comparison_histogram_plot(
                    j, self.approximant, param_samples, self.colors,
                    latex_labels[j],
                    approximant_labels=self.label_to_prepend_approximant)
                plt.savefig(self.savedir + "combined_1d_posterior_%s" % (j))
                plt.close()
                fig = plot._1d_cdf_comparison_plot(
                    j, self.approximant, param_samples, self.colors,
                    latex_labels[j],
                    approximant_labels=self.label_to_prepend_approximant)
                fig.savefig(self.savedir + "combined_cdf_%s" % (j))
                plt.close()
            except Exception as e:
                logger.info("Failed to generate comparison plots for %s "
                            "because %s" % (j, e))
                continue

    def _skymap_comparison_plot(self, idx="all"):
        """Generate a comparison skymap plot.

        Parameters
        ----------
        idx: int, optional
            The indicies of the results files that you wish to be included
            in the comparsion plots.
        """
        ind_ra = [i.index("ra") for i in self.parameters]
        ind_dec = [i.index("dec") for i in self.parameters]
        ra_list = [[k[ind_ra[num]] for k in l] for num, l in
                   enumerate(self.samples)]
        dec_list = [[k[ind_dec[num]] for k in l] for num, l in
                    enumerate(self.samples)]
        fig = plot._sky_map_comparison_plot(
            ra_list, dec_list, self.approximant, self.colors,
            approximant_labels=self.label_to_prepend_approximant)
        fig.savefig(self.savedir + "combined_skymap.png")
        plt.close()

    def _waveform_comparison_plot(self, idx="all"):
        """Generate a plot to compare waveforms as seen in the Hanford
        detector.

        Parameters
        ----------
        idx: int, optional
            The indicies of the results files that you wish to be included
            in the comparsion plots.
        """
        fig = plot._waveform_comparison_plot(
            self.maxL_samples, self.colors,
            approximant_labels=self.label_to_prepend_approximant)
        fig.savefig(self.savedir + "compare_waveforms.png")
        plt.close()
        fig = plot._time_domain_waveform_comparison_plot(
            self.maxL_samples, self.colors,
            approximant_labels=self.label_to_prepend_approximant)
        fig.savefig(self.savedir + "compare_time_domain_waveforms.png")
        plt.close()


def main():
    """Top level interface for `summaryplots`
    """
    parser = command_line()
    opts = parser.parse_args()
    inputs = Input(opts)
    PlotGeneration(inputs)


if __name__ == '__main__':
    main()



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import configparser
import shutil

import json
import h5py
import deepdish

import numpy as np

from pesummary.command_line import command_line
import pesummary.file.conversions as con
from pesummary.utils.utils import logger
from pesummary.file.lalinference import LALInferenceResultsFile
from pesummary.file.standard_names import standard_names

try:
    from glue.ligolw import ligolw
    from glue.ligolw import lsctables
    from glue.ligolw import utils as ligolw_utils
    GLUE = True
except ImportError:
    GLUE = False


def paths_to_key(key, dictionary, current_path=None):
    """Return the path to a key stored in a nested dictionary

    Parameters
    ----------
    key: str
        the key that you would like to find
    dictionary: dict
        the nested dictionary that has the key stored somewhere within it
    current_path: str, optional
        the current level in the dictionary
    """
    if current_path is None:
        current_path = []

    for k, v in dictionary.items():
        if k == key:
            yield current_path + [key]
        else:
            if isinstance(v, dict):
                path = current_path + [k]
                yield from paths_to_key(key, v, path)


def load_recusively(key, dictionary):
    """Return the entry for a key of format 'a/b/c/d'

    Parameters
    ----------
    key: str
        key of format 'a/b/c/d'
    dictionary: dict
        the dictionary that has the key stored
    """
    if "/" in key:
        key = key.split("/")
    if isinstance(key, str):
        key = [key]
    if key[-1] in dictionary.keys():
        yield dictionary[key[-1]]
    else:
        old, new = key[0], key[1:]
        yield from load_recusively(new, dictionary[old])


class OneFormat(object):
    """Class to convert a given results file into a standard format with all
    derived posterior distributions included

    Parameters
    ----------
    fil: str
       path to the results file
    inj: str, optional
       path to the file containing injection information
    config: str, optional
       path to the configuration file

    Attributes
    ----------
    extension: str
        the extension of the input file
    lalinference_hdf5_format: Bool
        Boolean determining if the hdf5 file is of LALInference format
    bilby_hdf5_format: Bool
        Boolean determining if the hdf5 file is of Bilby format
    data: list
        list containing the extracted data from the input file
    parameters: list
        list of parameters stored in the input file
    samples: list
        list of samples stored in the input file
    approximant: str
        the approximant stored in the input file
    """
    def __init__(self, fil, inj=None, config=None):
        logger.info("Extracting the information from %s" % (fil))
        self.fil = fil
        self.inj = inj
        self.config = config
        self.data = None
        self.injection_data = None

    @staticmethod
    def _check_definition_of_inclination(parameters):
        """Check the definition of inclination given the other parameters

        Parameters
        ----------
        parameters: list
            list of parameters used in the study
        """
        theta_jn = False
        spin_angles = ["tilt_1", "tilt_2", "a_1", "a_2"]
        names = [
            standard_names[i] for i in parameters if i in standard_names.keys()]
        if all(i in names for i in spin_angles):
            theta_jn = True
        if theta_jn:
            if "theta_jn" not in names and "inclination" in parameters:
                logger.warn("Because the spin angles are in your list of "
                            "parameters, the angle 'inclination' probably "
                            "refers to 'theta_jn'. If this is a mistake, "
                            "please change the definition of 'inclination' to "
                            "'iota' in your results file")
                index = parameters.index("inclination")
                parameters[index] = "theta_jn"
        else:
            if "inclination" in parameters:
                index = parameters.index("inclination")
                parameters[index] = "iota"
        return parameters

    @property
    def config(self):
        return self._config

    @config.setter
    def config(self, config):
        self._config = None
        self.fixed_data = None
        self.marg_par = None
        if config:
            data = self._extract_data_from_config_file(config)
            self.fixed_data = data["fixed_data"]
            self.marg_par = data["marginalized_parameters"]
            self._config = config

    @property
    def extension(self):
        return self.fil.split(".")[-1]

    @property
    def lalinference_hdf5_format(self):
        f = h5py.File(self.fil)
        keys = list(f.keys())
        f.close()
        if "lalinference" in keys:
            return True
        return False

    @property
    def bilby_hdf5_format(self):
        f = h5py.File(self.fil)
        keys = list(f.keys())
        f.close()
        if "data" in keys or "posterior" in keys:
            return True
        return False

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, data):
        func_map = {"json": self._grab_data_from_json_file,
                    "hdf5": self._grab_data_from_hdf5_file,
                    "h5": self._grab_data_from_hdf5_file,
                    "dat": self._grab_data_from_dat_file,
                    "txt": self._grab_data_from_dat_file}
        data = func_map[self.extension]()
        parameters = data[0]
        samples = data[1]

        if self.fixed_data:

            for i in self.fixed_data.keys():
                fixed_parameter = i
                fixed_value = self.fixed_data[i]

                try:
                    param = standard_names[fixed_parameter]
                    if param in parameters:
                        pass
                    else:
                        parameters.append(param)
                        for num in range(len(samples)):
                            samples[num].append(float(fixed_value))
                except Exception:
                    if fixed_parameter == "logdistance":
                        if "luminosity_distance" not in parameters:
                            parameters.append(standard_names["distance"])
                            for num in range(len(samples)):
                                samples[num].append(float(fixed_value))
                    if fixed_parameter == "costheta_jn":
                        if "theta_jn" not in parameters:
                            parameters.append(standard_names["theta_jn"])
                            for num in range(len(samples)):
                                samples[num].append(float(fixed_value))

        if self.marg_par:
            for i in self.marg_par.keys():
                if "time" in i and "geocent_time" not in parameters:
                    if "marginalized_geocent_time" in parameters:
                        ind = parameters.index("marginalized_geocent_time")
                        parameters.remove(parameters[ind])
                        parameters.append("geocent_time")
                        for num, j in enumerate(samples):
                            samples[num].append(float(j[ind]))
                            del j[ind]
                    else:
                        logger.warn("You have marginalized over time and "
                                    "there are no time samples. Manually "
                                    "setting time to 100000s")
                        parameters.append("geocent_time")
                        for num, j in enumerate(samples):
                            samples[num].append(float(100000))
                if "phi" in i and "phase" not in parameters:
                    if "marginalized_phase" in parameters:
                        ind = parameters.index("marginalized_phase")
                        parameters.remove(parameters[ind])
                        parameters.append("phase")
                        for num, j in enumerate(samples):
                            samples[num].append(float(j[ind]))
                            del j[ind]
                    else:
                        logger.warn("You have marginalized over phase and "
                                    "there are no phase samples. Manually "
                                    "setting the phase to be 0")
                        parameters.append("phase")
                        for num, j in enumerate(samples):
                            samples[num].append(float(0))
                if "dist" in i and "luminosity_distance" not in parameters:
                    if "marginalized_distance" in parameters:
                        ind = parameters.index("marginalized_distance")
                        parameters.remove(parameters[ind])
                        parameters.append("luminosity_distance")
                        for num, j in enumerate(samples):
                            samples[num].append(float(j[ind]))
                            del j[ind]
                    else:
                        logger.warn("You have marginalized over distance and "
                                    "there are no distance samples. Manually "
                                    "setting distance to 100Mpc")
                        parameters.append("luminosity_distance")
                        for num, j in enumerate(samples):
                            samples[num].append(float(100.0))

        if len(data) > 2:
            self._data = [parameters, samples, data[2]]
        else:
            self._data = [parameters, samples]

    @property
    def parameters(self):
        return self.data[0]

    @property
    def samples(self):
        return self.data[1]

    @property
    def approximant(self):
        if len(self.data) > 2:
            return self.data[2]
        return "none"

    @property
    def injection_data(self):
        return self._injection_data

    @injection_data.setter
    def injection_data(self, injection_data):
        if self.inj:
            extension = self.inj.split(".")[-1]
            func_map = {"xml": self._grab_injection_data_from_xml_file,
                        "hdf5": self._grab_injection_data_from_hdf5_file,
                        "h5": self._grab_injection_data_from_hdf5_file}
            self._injection_data = func_map[extension]()
        else:
            self._injection_data = [
                self.parameters, [float("nan")] * len(self.parameters)]

    @property
    def injection_parameters(self):
        return self.injection_data[0]

    @property
    def injection_values(self):
        return self.injection_data[1]

    def _grab_data_from_hdf5_file(self):
        """Grab the data stored in an hdf5 file
        """
        self._data_structure = []
        if self.lalinference_hdf5_format:
            return self._grab_data_with_h5py()
        elif self.bilby_hdf5_format:
            try:
                return self._grab_data_with_deepdish()
            except Exception as e:
                logger.warning("Failed to open %s with deepdish because %s. "
                               "Trying to grab the data with 'h5py'." % (
                                   self.fil, e))
                return self._grab_data_with_h5py()
        else:
            logger.warning("Unrecognised HDF5 format. Trying to open and find "
                           "the data")
            try:
                return self._grab_data_with_h5py()
            except Exception:
                raise Exception("Failed to extract the data from the results "
                                "file. Please reformat to either the bilby "
                                "or LALInference format")

    def _add_to_list(self, item):
        self._data_structure.append(item)

    def _grab_data_with_h5py(self):
        """Grab the data stored in an hdf5 file using h5py
        """
        samples = []
        f = h5py.File(self.fil)
        f.visit(self._add_to_list)
        for i in self._data_structure:
            condition1 = "posterior_samples" in i or "posterior"in i
            condition2 = "posterior_samples/" not in i and "posterior/" not in i
            if condition1 and condition2:
                path = i
        f.close()
        if self.lalinference_hdf5_format:
            g = LALInferenceResultsFile(self.fil)
            parameters, samples = g.grab_samples()
        elif self.bilby_hdf5_format:
            f = h5py.File(self.fil)
            parameters, data = [], []
            blocks = [i for i in f["%s" % (path)] if "block" in i]
            for i in blocks:
                block_name = i.split("_")[0]
                if "items" in i:
                    for par in f["%s/%s" % (path, i)]:
                        if par == b"waveform_approximant":
                            blocks.remove(block_name + "_items")
                            blocks.remove(block_name + "_values")
            for i in sorted(blocks):
                if "items" in i:
                    for par in f["%s/%s" % (path, i)]:
                        if par == b"logL":
                            parameters.append(b"log_likelihood")
                        else:
                            parameters.append(par)
                if "values" in i:
                    if len(data) == 0:
                        for dat in f["%s/%s" % (path, i)]:
                            data.append(list(np.real(dat)))
                    else:
                        for num, dat in enumerate(f["%s/%s" % (path, i)]):
                            data[num] += list(np.real(dat))
            parameters = [i.decode("utf-8") for i in parameters]
            f.close()
        return parameters, samples

    def _grab_data_with_deepdish(self):
        """Grab the data stored in an hdf5 file using deepdish
        """
        approx = "none"
        f = deepdish.io.load(self.fil)
        path, = paths_to_key("posterior", f)
        path = path[0]
        reduced_f, = load_recusively(path, f)
        parameters = [i for i in reduced_f.keys()]
        if "waveform_approximant" in parameters:
            approx = reduced_f["waveform_approximant"][0]
            parameters.remove("waveform_approximant")
        data = np.zeros([len(reduced_f[parameters[0]]), len(parameters)])
        for num, par in enumerate(parameters):
            for key, i in enumerate(reduced_f[par]):
                data[key][num] = float(np.real(i))
        data = data.tolist()
        for num, par in enumerate(parameters):
            if par == "logL":
                parameters[num] = "log_likelihood"
        return parameters, data, approx

    def _grab_data_from_json_file(self):
        """Grab the data stored in a .json file
        """
        with open(self.fil) as f:
            data = json.load(f)
        path, = paths_to_key("posterior", data)
        path = path[0]
        if "content" in data[path].keys():
            path += "/content"
        reduced_data, = load_recusively(path, data)
        parameters = list(reduced_data.keys())
        parameters = [standard_names[i] for i in list(reduced_data.keys()) if i
                      in standard_names.keys()]

        path_to_approximant = [
            i for i in paths_to_key("waveform_approximant", reduced_data)]
        try:
            approximant, = load_recusively("/".join(path_to_approximant[0]),
                                           data)
        except Exception:
            approximant = "none"

        samples = [[
            reduced_data[j][i] if not isinstance(reduced_data[j][i], dict)
            else reduced_data[j][i]["real"] for j in parameters] for i in
            range(len(reduced_data[parameters[0]]))]
        return parameters, samples, approximant

    def _grab_data_from_dat_file(self):
        """Grab the data stored in a .dat file
        """
        dat_file = np.genfromtxt(self.fil, names=True)
        stored_parameters = [i for i in dat_file.dtype.names]
        stored_parameters = self._check_definition_of_inclination(
            stored_parameters)
        parameters = [
            i for i in stored_parameters if i in standard_names.keys()]
        indices = [stored_parameters.index(i) for i in parameters]
        parameters = [standard_names[i] for i in parameters]
        samples = [[x[i] for i in indices] for x in dat_file]

        condition1 = "luminosity" not in parameters
        condition2 = "logdistance" in stored_parameters
        if condition1 and condition2:
            parameters.append("luminosity_distance")
            for num, i in enumerate(dat_file):
                samples[num].append(
                    np.exp(i[stored_parameters.index("logdistance")]))

        return parameters, samples

    def _extract_data_from_config_file(self, cp):
        """Grab the data from a config file
        """
        config = configparser.ConfigParser()
        try:
            config.read(cp)
            fixed_data = None
            marg_par = None
            if "engine" in config.sections():
                fixed_data = {
                    key.split("fix-")[1]: item for key, item in
                    config.items("engine") if "fix" in key}
                marg_par = {
                    key.split("marg")[1]: item for key, item in
                    config.items("engine") if "marg" in key}
            return {"fixed_data": fixed_data,
                    "marginalized_parameters": marg_par}
        except Exception:
            return {"fixed_data": None,
                    "marginalized_parameters": None}

    def _grab_injection_data_from_xml_file(self):
        """Grab the data from an xml injection file
        """
        func_map = {
            "chirp_mass": lambda inj: inj.mchirp,
            "luminosity_distance": lambda inj: inj.distance,
            "mass_1": lambda inj: inj.mass1,
            "mass_2": lambda inj: inj.mass2,
            "dec": lambda inj: inj.latitude,
            "spin_1x": lambda inj: inj.spin1x,
            "spin_1y": lambda inj: inj.spin1y,
            "spin_1z": lambda inj: inj.spin1z,
            "spin_2x": lambda inj: inj.spin2x,
            "spin_2y": lambda inj: inj.spin2y,
            "spin_2z": lambda inj: inj.spin2z,
            "mass_ratio": lambda inj: con.q_from_m1_m2(inj.mass1, inj.mass2),
            "symmetric_mass_ratio": lambda inj: con.eta_from_m1_m2(
                inj.mass1, inj.mass2),
            "total_mass": lambda inj: inj.mass1 + inj.mass2,
            "chi_p": lambda inj: con._chi_p(
                inj.mass1, inj.mass2, inj.spin1x, inj.spin1y, inj.spin2x,
                inj.spin2y),
            "chi_eff": lambda inj: con._chi_eff(inj.mass1, inj.mass2,
                                                inj.spin1z, inj.spin2z)}
        injection_parameters = self.parameters
        if GLUE:
            xmldoc = ligolw_utils.load_filename(
                self.inj, contenthandler=lsctables.use_in(
                    ligolw.LIGOLWContentHandler))
            table = lsctables.SimInspiralTable.get_table(xmldoc)[0]
            injection_values = [
                func_map[i](table) if i in func_map.keys() else float("nan")
                for i in self.parameters]
        else:
            injection_values = [float("nan")] * len(self.parameters)
        return injection_parameters, injection_values

    def _specific_parameter_samples(self, param):
        """Return the samples for a specific parameter

        Parameters
        ----------
        param: str
            the parameter that you would like to return the samples for
        """
        ind = self.parameters.index(param)
        samples = np.array([i[ind] for i in self.samples])
        return samples

    def specific_parameter_samples(self, param):
        """Return the samples for either a list or a single parameter

        Parameters
        ----------
        param: list/str
            the parameter/parameters that you would like to return the samples
            for
        """
        if type(param) == list:
            samples = [self._specific_parameter_samples(i) for i in param]
        else:
            samples = self._specific_parameter_samples(param)
        return samples

    def append_data(self, samples):
        """Add a list of samples to the existing samples data object

        Parameters
        ----------
        samples: list
            the list of samples that you would like to append
        """
        for num, i in enumerate(self.samples):
            self.samples[num].append(samples[num])

    def _mchirp_from_mchirp_source_z(self):
        self.parameters.append("chirp_mass")
        samples = self.specific_parameter_samples(["chirp_mass_source", "redshift"])
        chirp_mass = con.mchirp_from_mchirp_source_z(samples[0], samples[1])
        self.append_data(chirp_mass)

    def _q_from_eta(self):
        self.parameters.append("mass_ratio")
        samples = self.specific_parameter_samples("symmetric_mass_ratio")
        mass_ratio = con.q_from_eta(samples)
        self.append_data(mass_ratio)

    def _q_from_m1_m2(self):
        self.parameters.append("mass_ratio")
        samples = self.specific_parameter_samples(["mass_1", "mass_2"])
        mass_ratio = con.q_from_m1_m2(samples[0], samples[1])
        self.append_data(mass_ratio)

    def _invert_q(self):
        ind = self.parameters.index("mass_ratio")
        for num, i in enumerate(self.samples):
            self.samples[num][ind] = 1. / self.samples[num][ind]

    def _mchirp_from_mtotal_q(self):
        self.parameters.append("chirp_mass")
        samples = self.specific_parameter_samples(["total_mass", "mass_ratio"])
        chirp_mass = con.mchirp_from_mtotal_q(samples[0], samples[1])
        self.append_data(chirp_mass)

    def _m1_from_mchirp_q(self):
        self.parameters.append("mass_1")
        samples = self.specific_parameter_samples(["chirp_mass", "mass_ratio"])
        mass_1 = con.m1_from_mchirp_q(samples[0], samples[1])
        self.append_data(mass_1)

    def _m2_from_mchirp_q(self):
        self.parameters.append("mass_2")
        samples = self.specific_parameter_samples(["chirp_mass", "mass_ratio"])
        mass_2 = con.m2_from_mchirp_q(samples[0], samples[1])
        self.append_data(mass_2)

    def _reference_frequency(self):
        self.parameters.append("reference_frequency")
        nsamples = len(self.samples)
        self.append_data([20.] * nsamples)

    def _mtotal_from_m1_m2(self):
        self.parameters.append("total_mass")
        samples = self.specific_parameter_samples(["mass_1", "mass_2"])
        m_total = con.m_total_from_m1_m2(samples[0], samples[1])
        self.append_data(m_total)

    def _mchirp_from_m1_m2(self):
        self.parameters.append("chirp_mass")
        samples = self.specific_parameter_samples(["mass_1", "mass_2"])
        chirp_mass = con.m_total_from_m1_m2(samples[0], samples[1])
        self.append_data(chirp_mass)

    def _eta_from_m1_m2(self):
        self.parameters.append("symmetric_mass_ratio")
        samples = self.specific_parameter_samples(["mass_1", "mass_2"])
        eta = con.eta_from_m1_m2(samples[0], samples[1])
        self.append_data(eta)

    def _phi_12_from_phi1_phi2(self):
        self.parameters.append("phi_12")
        samples = self.specific_parameter_samples(["phi_1", "phi_2"])
        phi_12 = con.phi_12_from_phi1_phi2(samples[0], samples[1])
        self.append_data(phi_12)

    def _spin_angles(self):
        spin_angles = ["theta_jn", "phi_jl", "tilt_1", "tilt_2", "phi_12",
                       "a_1", "a_2"]
        spin_angles_to_calculate = [
            i for i in spin_angles if i not in self.parameters]
        for i in spin_angles_to_calculate:
            self.parameters.append(i)
        spin_components = [
            "mass_1", "mass_2", "iota", "spin_1x", "spin_1y", "spin_1z",
            "spin_2x", "spin_2y", "spin_2z", "reference_frequency", "phase"]
        samples = self.specific_parameter_samples(spin_components)
        spin_angles = con.spin_angles(
            samples[0], samples[1], samples[2], samples[3], samples[4],
            samples[5], samples[6], samples[7], samples[8], samples[9],
            samples[10])

        for i in spin_angles_to_calculate:
            ind = spin_angles_to_calculate.index(i)
            data = np.array([i[ind] for i in spin_angles])
            self.append_data(data)

    def _component_spins(self):
        spins = ["iota", "spin_1x", "spin_1y", "spin_1z", "spin_2x", "spin_2y",
                 "spin_2z"]
        spins_to_calculate = [
            i for i in spins if i not in self.parameters]
        for i in spins_to_calculate:
            self.parameters.append(i)
        spin_angles = [
            "theta_jn", "phi_jl", "tilt_1", "tilt_2", "phi_12", "a_1", "a_2",
            "mass_1", "mass_2", "reference_frequency", "phase"]
        samples = self.specific_parameter_samples(spin_angles)
        spin_components = con.component_spins(
            samples[0], samples[1], samples[2], samples[3], samples[4],
            samples[5], samples[6], samples[7], samples[8], samples[9],
            samples[10])

        for i in spins_to_calculate:
            ind = spins.index(i)
            data = np.array([i[ind] for i in spin_components])
            self.append_data(data)

    def _chi_p(self):
        self.parameters.append("chi_p")
        parameters = [
            "mass_1", "mass_2", "spin_1x", "spin_1y", "spin_2x", "spin_2y"]
        samples = self.specific_parameter_samples(parameters)
        chi_p_samples = con.chi_p(
            samples[0], samples[1], samples[2], samples[3], samples[4],
            samples[5])
        self.append_data(chi_p_samples)

    def _chi_eff(self):
        self.parameters.append("chi_eff")
        parameters = ["mass_1", "mass_2", "spin_1z", "spin_2z"]
        samples = self.specific_parameter_samples(parameters)
        chi_eff_samples = con.chi_eff(
            samples[0], samples[1], samples[2], samples[3])
        self.append_data(chi_eff_samples)

    def _cos_tilt_1_from_tilt_1(self):
        self.parameters.append("cos_tilt_1")
        samples = self.specific_parameter_samples("tilt_1")
        cos_tilt_1 = np.cos(samples)
        self.append_data(cos_tilt_1)

    def _cos_tilt_2_from_tilt_2(self):
        self.parameters.append("cos_tilt_2")
        samples = self.specific_parameter_samples("tilt_2")
        cos_tilt_2 = np.cos(samples)
        self.append_data(cos_tilt_2)

    def _dL_from_z(self):
        self.parameters.append("luminosity_distance")
        samples = self.specific_parameter_samples("redshift")
        distance = con.dL_from_z(samples)
        self.append_data(distance)

    def _z_from_dL(self):
        self.parameters.append("redshift")
        samples = self.specific_parameter_samples("luminosity_distance")
        redshift = con.z_from_dL(samples)
        self.append_data(redshift)

    def _comoving_distance_from_z(self):
        self.parameters.append("comoving_distance")
        samples = self.specific_parameter_samples("redshift")
        distance = con.comoving_distance_from_z(samples)
        self.append_data(distance)

    def _m1_source_from_m1_z(self):
        self.parameters.append("mass_1_source")
        samples = self.specific_parameter_samples(["mass_1", "redshift"])
        mass_1_source = con.m1_source_from_m1_z(samples[0], samples[1])
        self.append_data(mass_1_source)

    def _m2_source_from_m2_z(self):
        self.parameters.append("mass_2_source")
        samples = self.specific_parameter_samples(["mass_2", "redshift"])
        mass_2_source = con.m2_source_from_m2_z(samples[0], samples[1])
        self.append_data(mass_2_source)

    def _mtotal_source_from_mtotal_z(self):
        self.parameters.append("total_mass_source")
        samples = self.specific_parameter_samples(["total_mass", "redshift"])
        total_mass_source = con.m_total_source_from_mtotal_z(samples[0], samples[1])
        self.append_data(total_mass_source)

    def _mchirp_source_from_mchirp_z(self):
        self.parameters.append("chirp_mass_source")
        samples = self.specific_parameter_samples(["chirp_mass", "redshift"])
        chirp_mass_source = con.mchirp_source_from_mchirp_z(samples[0], samples[1])
        self.append_data(chirp_mass_source)

    def generate_all_posterior_samples(self):
        logger.debug("Starting to generate all derived posteriors")
        if "chirp_mass" not in self.parameters and "chirp_mass_source" in \
                self.parameters and "redshift" in self.parameters:
            self._mchirp_from_mchirp_source_z()
        if "mass_ratio" not in self.parameters and "symmetric_mass_ratio" in \
                self.parameters:
            self._q_from_eta()
        if "mass_ratio" not in self.parameters and "mass_1" in self.parameters \
                and "mass_2" in self.parameters:
            self._q_from_m1_m2()
        if "mass_ratio" in self.parameters:
            ind = self.parameters.index("mass_ratio")
            median = np.median([i[ind] for i in self.samples])
            if median < 1.:
                self._invert_q()
        if "chirp_mass" not in self.parameters and "total_mass" in self.parameters:
            self._mchirp_from_mtotal_q()
        if "mass_1" not in self.parameters and "chirp_mass" in self.parameters:
            self._m1_from_mchirp_q()
        if "mass_2" not in self.parameters and "chirp_mass" in self.parameters:
            self._m2_from_mchirp_q()
        if "reference_frequency" not in self.parameters:
            self._reference_frequency()

        condition1 = "phi_12" not in self.parameters
        condition2 = "phi_1" in self.parameters and "phi_2" in self.parameters
        if condition1 and condition2:
            self._phi_12_from_phi1_phi2()
        if "mass_1" in self.parameters and "mass_2" in self.parameters:
            if "total_mass" not in self.parameters:
                self._mtotal_from_m1_m2()
            if "chirp_mass" not in self.parameters:
                self._mchirp_from_m1_m2()
            if "symmetric_mass_ratio" not in self.parameters:
                self._eta_from_m1_m2()
            spin_components = [
                "spin_1x", "spin_1y", "spin_1z", "spin_2x", "spin_2y", "spin_2z"]
            spin_angles = [
                "theta_jn", "phi_jl", "tilt_1", "tilt_2", "phi_12", "a_1",
                "a_2", "mass_1", "mass_2", "reference_frequency", "phase"]
            if all(i in self.parameters for i in spin_components):
                self._spin_angles()
            if all(i in self.parameters for i in spin_angles):
                self._component_spins()
            if "chi_p" not in self.parameters and "chi_eff" not in self.parameters:
                if all(i in self.parameters for i in spin_angles):
                    self._chi_p()
                    self._chi_eff()
        if "cos_tilt_1" not in self.parameters and "tilt_1" in self.parameters:
            self._cos_tilt_1_from_tilt_1()
        if "cos_tilt_2" not in self.parameters and "tilt_2" in self.parameters:
            self._cos_tilt_2_from_tilt_2()
        if "luminosity_distance" not in self.parameters and "redshift" in self.parameters:
            self._dL_from_z()
        if "redshift" not in self.parameters and "luminosity_distance" in self.parameters:
            self._z_from_dL()
        if "comoving_distance" not in self.parameters and "redshift" in self.parameters:
            self._comoving_distance_from_z()
        if "redshift" in self.parameters:
            if "mass_1_source" not in self.parameters and "mass_1" in self.parameters:
                self._m1_source_from_m1_z()
            if "mass_2_source" not in self.parameters and "mass_2" in self.parameters:
                self._m2_source_from_m2_z()
            if "total_mass_source" not in self.parameters and "total_mass" in self.parameters:
                self._mtotal_source_from_mtotal_z()
            if "chirp_mass_source" not in self.parameters and "chirp_mass" in self.parameters:
                self._mchirp_source_from_mchirp_z()
        if "reference_frequency" in self.parameters:
            ind = self.parameters.index("reference_frequency")
            self.parameters.remove(self.parameters[ind])
            for i in self.samples:
                del i[ind]
        if "minimum_frequency" in self.parameters:
            ind = self.parameters.index("minimum_frequency")
            self.parameters.remove(self.parameters[ind])
            for i in self.samples:
                del i[ind]
        if "logPrior" in self.parameters:
            ind = self.parameters.index("logPrior")
            self.parameters.remove(self.parameters[ind])
            for i in self.samples:
                del i[ind]
        if "log_prior" in self.parameters:
            ind = self.parameters.index("log_prior")
            self.parameters.remove(self.parameters[ind])
            for i in self.samples:
                del i[ind]
        self._update_injection_data()

    def _update_injection_data(self):
        if self.inj:
            extension = self.inj.split(".")[-1]
            func_map = {"xml": self._grab_injection_data_from_xml_file,
                        "hdf5": self._grab_injection_data_from_hdf5_file,
                        "h5": self._grab_injection_data_from_hdf5_file}
            self._injection_data = func_map[extension]()
        else:
            self._injection_data = [
                self.parameters, [float("nan")] * len(self.parameters)]

    def save(self):
        parameters = np.array(self.parameters, dtype="S")
        injection_parameters = np.array(self.injection_parameters, dtype="S")
        injection_data = np.array(self.injection_values)
        f = h5py.File("%s_temp" % (self.fil), "w")
        posterior_samples_group = f.create_group("posterior_samples")
        label_group = posterior_samples_group.create_group("label")
        group = label_group.create_group(self.approximant)
        group.create_dataset("parameter_names", data=parameters)
        group.create_dataset("samples", data=self.samples)
        group.create_dataset("injection_parameters", data=injection_parameters)
        group.create_dataset("injection_data", data=injection_data)
        f.close()
        return "%s_temp" % (self.fil)


def add_specific_arguments(parser):
    """Add command line arguments that are specific to pesummary_convert

    Parameters
    ----------
    parser: argparser
        The parser containing the command line arguments
    """
    parser.add_argument("-o", "--outpath", dest="out",
                        help="location of output file", default=None)
    return parser


def main():
    """Top-level interface for pesummary_convert.py
    """
    parser = command_line()
    parser = add_specific_arguments(parser)
    opts = parser.parse_args()
    if opts.inj_file and len(opts.samples) != len(opts.inj_file):
        raise Exception("Please ensure that the number of results files "
                        "matches the number of injection files")
    if opts.config and len(opts.samples) != len(opts.config):
        raise Exception("Please ensure that the number of results files "
                        "matches the number of configuration files")
    if not opts.inj_file:
        opts.inj_file = []
        for i in range(len(opts.samples)):
            opts.inj_file.append(None)
    if not opts.config:
        opts.config = []
        for i in range(len(opts.samples)):
            opts.config.append(None)
    for num, i in enumerate(opts.samples):
        f = OneFormat(i, opts.inj_file[num], config=opts.config[num])
        f.generate_all_posterior_samples()
        g = f.save()
        if opts.out:
            shutil.move(g, opts.out)



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import argparse


def command_line():
    """Creates an ArgumentParser object which holds all of the information
    from the command line.
    """
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("-w", "--webdir", dest="webdir",
                        help="make page and plots in DIR", metavar="DIR",
                        default=None)
    parser.add_argument("-b", "--baseurl", dest="baseurl",
                        help="make the page at this url", metavar="DIR",
                        default=None)
    parser.add_argument("-s", "--samples", dest="samples",
                        help="Posterior samples hdf5 file", nargs='+',
                        default=None)
    parser.add_argument("-a", "--approximant", dest="approximant",
                        help="waveform approximant used to generate samples",
                        nargs='+', default=None)
    parser.add_argument("--email", action="store",
                        help="send an e-mail to the given address with a link "
                             "to the finished page.",
                        default=None, metavar="user@ligo.org")
    parser.add_argument("--dump", action="store_true",
                        help="dump all information onto a single html page",
                        default=False)
    parser.add_argument("-c", "--config", dest="config",
                        help="configuration file associcated with each "
                             "samples file.", nargs='+', default=None)
    parser.add_argument("--sensitivity", action="store_true",
                        help="generate sky sensitivities for HL, HLV",
                        default=False)
    parser.add_argument("--add_to_existing", action="store_true",
                        help="add new results to an existing html page",
                        default=False)
    parser.add_argument("-e", "--existing_webdir", dest="existing",
                        help="web directory of existing output",
                        default=None)
    parser.add_argument("-i", "--inj_file", dest="inj_file",
                        help="path to injetcion file", nargs='+',
                        default=None)
    parser.add_argument("--user", dest="user", help=argparse.SUPPRESS,
                        default="albert.einstein")
    parser.add_argument("--gracedb", dest="gracedb",
                        help="gracedb of the event", default=None)
    parser.add_argument("--psd", dest="psd",
                        help="psd files used", nargs='+',
                        default=None)
    parser.add_argument("--calibration", dest="calibration",
                        help="files for the calibration envelope", nargs="+",
                        default=None)
    parser.add_argument("--labels", dest="labels",
                        help="labels used to distinguish runs", nargs='+',
                        default=None)
    parser.add_argument("-v", "--verbose", action="store_true",
                        help="print useful information for debugging purposes")
    parser.add_argument("--save_to_hdf5", action="store_true",
                        help="save the meta file in hdf5 format",
                        default=False)
    return parser



#! /usr/bin/env python

# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import socket
import os
import shutil
from glob import glob

import numpy as np
import h5py

import pesummary
from pesummary.utils.utils import (guess_url, logger,
                                   rename_group_or_dataset_in_hf5_file)
from pesummary.utils import utils
from pesummary.file.one_format import OneFormat
from pesummary.file.existing import ExistingFile
from pesummary.file.lalinference import LALInferenceResultsFile

__doc__ == "Classes to handle the command line inputs"


class Input(object):
    """Super class to handle command line arguments

    Parameters
    ----------
    parser: argparser
        The parser containing the command line arguments

    Attributes
    ----------
    user: str
        The user who submitted the job
    add_to_existing: Bool
        Boolean to determine if you wish to add to a existing webpage
    existing: str
        Existing web directory
    webdir: str
        Directory to output all information
    baseurl: str
        The url path for the corresponding web directory
    inj_file: List
        List containing paths to the injection file
    result_files: list
        List containing paths to the results files which are being analysed
    config: list
        List containing paths to the configuration files used to generate each
        results files
    approximant: list
        List of approximants used in the analysis to generate each results
        files
    email: str
        The email address to notify when the job has completed
    sensitivity: Bool
        Boolean to determine if you wish to plot the sky sensitivity for
        different detector networks
    gracedb: str
        The gracedb of the event that produced the results files
    dump: Bool
        Boolean to determine if you wish to produce a dumped html page layout
    detectors: list
        List containing the detectors used to generate each results file
    labels: str
        A label for this summary page
    psd: str
        List of the psds used in the analysis
    """
    def __init__(self, opts):
        logger.info("Command line arguments: %s" % (opts))
        self.user = opts.user
        self.existing = opts.existing
        self.webdir = opts.webdir
        self.baseurl = opts.baseurl
        self.inj_file = opts.inj_file
        self.config = opts.config
        self.result_files = opts.samples
        self.approximant = opts.approximant
        self.email = opts.email
        self.add_to_existing = opts.add_to_existing
        self.sensitivity = opts.sensitivity
        self.gracedb = opts.gracedb
        self.psds = opts.psd
        self.dump = opts.dump
        self.hdf5 = opts.save_to_hdf5
        self.existing_labels = []
        self.existing_parameters = []
        self.existing_samples = []
        self.existing_approximant = []
        self.existing_names = []
        self.check_approximant_in_results_file()
        self.make_directories()
        self.copy_files()
        self.detectors = None
        self.labels = opts.labels
        self.calibration = opts.calibration
        self.check_label_in_results_file()

    @property
    def user(self):
        return self._user

    @user.setter
    def user(self, user):
        try:
            self._user = os.environ["USER"]
            logger.debug("The following user submitted the job %s"
                         % (self._user))
        except Exception as e:
            logger.info("Failed to grab user information because %s. "
                        "Default will be used" % (e))
            self._user = user

    @property
    def host(self):
        return socket.getfqdn()

    @property
    def existing(self):
        return self._existing

    @existing.setter
    def existing(self, existing):
        if not existing:
            self._existing = None
        else:
            self._existing = existing

    @property
    def webdir(self):
        return self._webdir

    @webdir.setter
    def webdir(self, webdir):
        if not webdir and not self.existing:
            raise Exception("Please provide a web directory to store the "
                            "webpages. If this is an existing directory "
                            "pass this path under the --existing_dir "
                            "argument. If this is a new directory then "
                            "pass this under the --webdir argument")
        elif not webdir and self.existing:
            if not os.path.isdir(self.existing):
                raise Exception("The directory %s does not "
                                "exist" % (self.existing))
            entries = glob(self.existing + "/*")
            if "%s/home.html" % (self.existing) not in entries:
                raise Exception("Please give the base directory of an "
                                "existing output")
            self._webdir = self.existing
        if webdir:
            if not os.path.isdir(webdir):
                logger.debug("Given web directory does not exist. "
                             "Creating it now")
                utils.make_dir(webdir)
            self._webdir = webdir

    @property
    def baseurl(self):
        return self._baseurl

    @baseurl.setter
    def baseurl(self, baseurl):
        self._baseurl = baseurl
        if not baseurl:
            self._baseurl = guess_url(self.webdir, socket.getfqdn(), self.user)
            logger.debug("No url is provided. The url %s will be "
                         "used" % (self._baseurl))

    @property
    def inj_file(self):
        return self._inj_file

    @inj_file.setter
    def inj_file(self, inj_file):
        if inj_file:
            self._inj_file = inj_file
        else:
            self._inj_file = None
        self._inj_file = inj_file

    @property
    def result_files(self):
        return self._samples

    @result_files.setter
    def result_files(self, samples):
        sample_list = []
        if not samples:
            raise Exception("Please provide a results file")
        if self.inj_file and len(samples) != len(self.inj_file):
            raise Exception("The number of results files does not match the "
                            "number of injection files")
        if not self.inj_file:
            self.inj_file = [None] * len(samples)
        for num, i in enumerate(samples):
            if not os.path.isfile(i):
                raise Exception("File %s does not exist" % (i))
            config = None
            if self.config and len(samples) != len(self.config):
                raise Exception("Ensure that the number of results files "
                                "match the number of configuration files")
            if self.config:
                config = self.config[num]
            std_form = self.convert_to_standard_format(i, self.inj_file[num],
                                                       config_file=config)
            sample_list.append(std_form)
        self._samples = sample_list
    """
    @property
    def config(self):
        return self._config

    @config.setter
    def config(self, config):
        if config and len(self.result_files) != len(config):
            raise Exception("Ensure that the number of results files match "
                            "the number of configuration files")
        self._config = config
    """

    @property
    def approximant(self):
        return self._approximant

    @approximant.setter
    def approximant(self, approximant):
        if not approximant:
            logger.debug("No approximant is given. Trying to extract from "
                         "results file")
            approximant_list = []
            for i in self.result_files:
                f = h5py.File(i, "r")
                approx = list(f["posterior_samples/label"].keys())[0]
                f.close()
                if approx == "none":
                    raise Exception("Failed to extract the approximant "
                                    "from the file: %s. Please pass the "
                                    "approximant with the flag "
                                    "--approximant" % (i.split("_temp")[0]))
                approximant_list.append(approx)
        else:
            approximant_list = approximant
        if len(approximant_list) != len(self.result_files):
            raise Exception("The number of results files does not match the "
                            "number of approximants")
        self._approximant = approximant_list

    @property
    def email(self):
        return self._email

    @email.setter
    def email(self, email):
        if email:
            self._email = email
        else:
            self._email = None

    @property
    def add_to_existing(self):
        return self._add_to_existing

    @add_to_existing.setter
    def add_to_existing(self, add_to_existing):
        self._add_to_existing = False
        if add_to_existing and not self.existing:
            raise Exception("Please provide a current html page that you "
                            "wish to add content to")
        if not add_to_existing and self.existing:
            logger.debug("Existing html page has been given without "
                         "specifying --add_to_existing flag. This is probably "
                         "and error and so manually adding --add_to_existing "
                         "flag")
            self._add_to_existing = True
        if add_to_existing and self.existing:
            if self.config:
                for i in glob(self.existing + "/config/*"):
                    self.config.append(i)
            self._add_to_existing = True

    @property
    def gracedb(self):
        return self._gracedb

    @gracedb.setter
    def gracedb(self, gracedb):
        if gracedb:
            self._gracedb = gracedb
        else:
            self._gracedb = None

    @property
    def dump(self):
        return self._dump

    @dump.setter
    def dump(self, dump):
        if dump:
            self._dump = True
        else:
            self._dump = False

    @property
    def detectors(self):
        return self._detectors

    @detectors.setter
    def detectors(self, detectors):
        detector_list = []
        if not detectors:
            for num, i in enumerate(self.result_files):
                f = h5py.File(i)
                path = ("posterior_samples/label/%s/parameter_names"
                        % (self.approximant[num]))
                params = [j for j in f[path]]
                f.close()
                individual_detectors = []
                for j in params:
                    if b"optimal_snr" in j:
                        det = j.split(b"_optimal_snr")[0]
                        individual_detectors.append(det)
                individual_detectors = sorted(
                    [str(i.decode("utf-8")) for i in individual_detectors])
                if individual_detectors:
                    detector_list.append("_".join(individual_detectors))
                else:
                    detector_list.append(None)
        else:
            detector_list = detectors
        logger.debug("The detector network is %s" % (detector_list))
        self._detectors = detector_list

    @property
    def existing_labels(self):
        return self._existing_labels

    @existing_labels.setter
    def existing_labels(self, existing_labels):
        self._existing_labels = None
        if self.add_to_existing:
            existing = ExistingFile(self.existing)
            self._existing_labels = existing.existing_labels

    @property
    def existing_parameters(self):
        return self._existing_parameters

    @existing_parameters.setter
    def existing_parameters(self, existing_parameters):
        self._existing_parameters = None
        if self.add_to_existing:
            existing = ExistingFile(self.existing)
            self._existing_parameters = existing.existing_parameters

    @property
    def existing_samples(self):
        return self._existing_samples

    @existing_samples.setter
    def existing_samples(self, existing_samples):
        self._existing_samples = None
        if self.add_to_existing:
            existing = ExistingFile(self.existing)
            self._existing_samples = existing.existing_samples

    @property
    def existing_approximant(self):
        return self._existing_approximant

    @existing_approximant.setter
    def existing_approximant(self, existing_approximant):
        self._existing_approximant = None
        if self.add_to_existing:
            existing = ExistingFile(self.existing)
            self._existing_approximant = existing.existing_approximant

    @property
    def existing_names(self):
        return self._existing_names

    @existing_names.setter
    def existing_names(self, existing_names):
        self._existing_names = None
        if self.add_to_existing:
            self._existing_names = [
                "%s_%s" % (i, j) for i, j in zip(
                    self.existing_labels, self.existing_approximant)]

    @property
    def existing_meta_file(self):
        if self.add_to_existing:
            existing = ExistingFile(self.existing)
            return existing.existing_file
        return None

    @property
    def labels(self):
        return self._labels

    @labels.setter
    def labels(self, labels):
        if labels is not None:
            if len(labels) != len(self.result_files):
                raise Exception(
                    "The number of labels does not match the number of results "
                    "files.")
            proposed_names = ["%s_%s" % (i, j) for i, j in zip(
                labels, self.approximant)]
            duplicates = dict(set(
                (x, proposed_names.count(x)) for x in
                filter(lambda rec: proposed_names.count(rec) > 1, proposed_names)))
            if len(duplicates.keys()) >= 1:
                raise Exception(
                    "The labels and approximant combination that you have "
                    "given do not give unique combinations. Please give other "
                    "labels")
            if self.add_to_existing:
                existing_names = [
                    "%s_%s" % (i, j) for i, j in zip(
                        self.existing_labels, self.existing_approximant)]
                for i in proposed_names:
                    if i in existing_names:
                        raise Exception(
                            "The labels and approximant combination that you "
                            "have given match those already in the existing "
                            "file. Please choose other labels")
            self._labels = labels
        else:
            label_list = self._default_labels()
            self._labels = label_list
        logger.debug("The label is %s" % (self._labels))

    @property
    def psds(self):
        return self._psds

    @psds.setter
    def psds(self, psds):
        psd_list = []
        if psds:
            for i in psds:
                extension = i.split(".")[-1]
                if extension == "gz":
                    print("convert to .dat file")
                elif extension == "dat":
                    psd_list.append(i)
                else:
                    raise Exception("PSD results file not understood")
            self._psds = psd_list
        else:
            self._psds = None

    @property
    def calibration(self):
        return self._calibration

    @calibration.setter
    def calibration(self, calibration):
        calibration_list = []
        if calibration:
            for i in calibration:
                f = np.genfromtxt(i)
                if len(f[0]) != 7:
                    raise Exception("Calibration envelope file not understood")
                calibration_list.append(f)
            self._calibration = calibration_list
            self.calibration_labels = [
                self._IFO_from_file_name(i) for i in calibration]
        else:
            logger.debug("No calibration envelope file given. Checking the "
                         "results file")
            self._calibration_envelope = False
            for i in self.result_files:
                try:
                    f = LALInferenceResultsFile(i.split("_temp")[0])
                    data, labels = f.grab_calibration_data()
                    if data == []:
                        logger.info("Failed to grab calibration data from %s" % (i))
                        data, labels = None, None
                except Exception:
                    logger.info("Failed to grab calibration data from %s" % (i))
                    data, labels = None, None
            self._calibration = data
            self.calibration_labels = labels

    @staticmethod
    def _IFO_from_file_name(file):
        """Return a guess of the IFO from the file name.

        Parameters
        ----------
        file: str
            the name of the file that you would like to make a guess for
        """
        file_name = file.split("/")[-1]
        if any(j in file_name for j in ["H", "_0", "IFO0"]):
            ifo = "H1"
        elif any(j in file_name for j in ["L", "_1", "IFO1"]):
            ifo = "L1"
        elif any(j in file_name for j in ["V", "_2", "IFO2"]):
            ifo = "V1"
        else:
            ifo = file_name
        return ifo

    def check_approximant_in_results_file(self):
        """Check that the approximant that is stored in the results file
        corresponds to the given approximant. If not then this will be changed.
        """
        for num, i in enumerate(self.result_files):
            f = h5py.File(i, "r")
            path = "posterior_samples/label"
            keys = list(f["posterior_samples/label"].keys())
            f.close()
            if "none" in keys:
                rename_group_or_dataset_in_hf5_file(
                    i, group=[
                        "%s/none" % (path),
                        "%s/%s" % (path, self.approximant[num])])

    def check_label_in_results_file(self):
        """Check that the label that is stored in the results file corresponds
        to the given label. If not then this will be changed.
        """
        for num, i in enumerate(self.result_files):
            rename_group_or_dataset_in_hf5_file(
                i, group=["posterior_samples/label",
                          "posterior_samples/%s" % (self.labels[num])])

    def make_directories(self):
        """Make the directorys in the web directory to store all information.
        """
        dirs = ["samples", "plots", "js", "html", "css", "plots/corner",
                "config"]
        for i in dirs:
            utils.make_dir(self.webdir + "/%s" % (i))

    def copy_files(self):
        """Copy the relevant files to the web directory.
        """
        logger.info("Copying the files to %s" % (self.webdir))
        path = pesummary.__file__[:-12]
        scripts = glob(path + "/js/*.js")
        for i in scripts:
            shutil.copyfile(i, self.webdir + "/js/%s" % (i.split("/")[-1]))
        scripts = glob(path + "/css/*.css")
        for i in scripts:
            shutil.copyfile(i, self.webdir + "/css/%s" % (i.split("/")[-1]))
        if self.config:
            for num, i in enumerate(self.config):
                if self.webdir not in i:
                    shutil.copyfile(i, self.webdir + "/config/"
                                    + self.approximant[num] + "_"
                                    + i.split("/")[-1])

    def convert_to_standard_format(self, results_file, injection_file=None,
                                   config_file=None):
        """Convert a results file to standard form.

        Parameters
        ----------
        results_file: str
            Path to the results file that you wish to convert to standard
            form
        injection_file: str, optional
            Path to the injection file that was used in the analysis to
            produce the results_file
        config_file: str, optional
            Path to the configuration file that was used
        """
        f = OneFormat(results_file, injection_file, config=config_file)
        f.generate_all_posterior_samples()
        return f.save()

    def _default_labels(self):
        """Return the defaut labels given your detector network.
        """
        label_list = []
        for num, i in enumerate(self.result_files):
            condition_list = type(self.detectors[num]) == list
            condition_str = type(self.detectors[num]) == str
            if self.gracedb and self.detectors and condition_list:
                for i in self.detectors[num]:
                    label_list.append("_".join([self.gracedb, i]))
            elif self.gracedb and self.detectors and condition_str:
                label_list.append("_".join(
                    [self.gracedb, self.detectors[num]]))
            elif self.gracedb:
                label_list.append(self.gracedb)
            elif self.detectors and type(self.detectors[num]) == list:
                for i in self.detectors[num]:
                    label_list.append("_".join(self.detectors[num]))
            elif self.detectors and type(self.detectors[num]) == str:
                label_list.append(self.detectors[num])
            else:
                label_list.append("%s" % (num))
        proposed_names = ["%s_%s" % (i, j) for i, j in zip(
            label_list, self.approximant)]
        duplicates = dict(set(
            (x, proposed_names.count(x)) for x in
            filter(lambda rec: proposed_names.count(rec) > 1, proposed_names)))
        for i in duplicates.keys():
            for j in range(duplicates[i]):
                ind = proposed_names.index(i)
                proposed_names[ind] += "_%s" % (j)
                label_list[ind] += "_%s" % (j)
        if self.add_to_existing:
            existing_names = [
                "%s_%s" % (i, j) for i, j in zip(
                    self.existing_labels, self.existing_approximant)]
            for num, i in enumerate(proposed_names):
                if i in existing_names:
                    ind = proposed_names.index(i)
                    label_list[ind] += "_%s" % (num)
        return label_list


class PostProcessing(object):
    """Class to extract parameters from the results files

    Parameters
    ----------
    inputs: argparser
        The parser containing the command line arguments
    colors: list, optional
        colors that you would like to use to display each results file in the
        webpage

    Attributes
    ----------
    parameters: list
        list of parameters that have posterior distributions for each results
        file
    injection_data: list
        list of dictionaries that contain the injection parameters and their
        injected value for each results file
    samples: list
        list of posterior samples for each parameter for each results file
    maxL_samples: list
        list of dictionaries that contain each parameter and their
        corresponding maximum likelihood value for each results file
    same_parameters: list
        List of parameters that all results files have sampled over
    """
    def __init__(self, inputs, colors="default"):
        self.inputs = inputs
        self.webdir = inputs.webdir
        self.baseurl = inputs.baseurl
        self.result_files = inputs.result_files
        self.approximant = inputs.approximant
        self.detectors = inputs.detectors
        self.dump = inputs.dump
        self.email = inputs.email
        self.user = inputs.user
        self.host = inputs.host
        self.config = inputs.config
        self.existing = inputs.existing
        self.gracedb = inputs.gracedb
        self.add_to_existing = inputs.add_to_existing
        self.labels = inputs.labels
        self.calibration = inputs.calibration
        self.calibration_labels = inputs.calibration_labels
        self.sensitivity = inputs.sensitivity
        self.psds = inputs.psds
        self.hdf5 = inputs.hdf5
        self.existing_labels = inputs.existing_labels
        self.existing_parameters = inputs.existing_parameters
        self.existing_samples = inputs.existing_samples
        self.existing_approximant = inputs.existing_approximant
        self.existing_names = inputs.existing_names
        self.existing_meta_file = inputs.existing_meta_file
        self.colors = colors

        self.grab_data_map = {"existing_file": self._data_from_existing_file,
                              "standard_format": self._data_from_standard_format}

        self.result_file_data = []
        self.maxL_samples = []
        self.same_parameters = []

    @property
    def coherence_test(self):
        duplicates = dict(set(
            (x, self.approximant.count(x)) for x in filter(
                lambda rec: self.approximant.count(rec) > 1,
                self.approximant)))
        if len(duplicates.keys()) > 0:
            return True
        return False

    @property
    def colors(self):
        return self._colors

    @colors.setter
    def colors(self, colors):
        if colors == "default":
            self._colors = ["#0173B2", "#DE8F05", "#029E73", "#D55E00",
                            "#CA9161", "#FBAFE4", "#949494", "#ECE133",
                            "#56B4E9"]
        else:
            if not len(self.result_files) <= len(colors):
                raise Exception("Please give the same number of colors as "
                                "results files")
            self._colors = colors

    @property
    def parameters(self):
        return self._parameters

    @property
    def result_file_data(self):
        return self._data

    @result_file_data.setter
    def result_file_data(self, data):
        data, parameters, samples, injection = [], [], [], []
        for num, result_file in enumerate(self.result_files):
            if result_file == self.existing_meta_file:
                key = "existing_file"
            else:
                key = "standard_format"
            data.append(self.grab_data_map[key](result_file, num))
        for i in data:
            if isinstance(i[0][0], list):
                for j in i[0]:
                    parameters.append(j)
                for j in i[1]:
                    samples.append(j)
                for j in i[2]:
                    injection.append(j)
            else:
                parameters.append(i[0])
                samples.append(i[1])
                injection.append(i[2])
        self._parameters = parameters
        self._samples = samples
        self._injection_data = injection
        self._data = data

    def _data_from_standard_format(self, result_file, index):
        """Extract data from a file of standard format

        Parameters
        ----------
        result_file: str
            path to the results file that you would like to extrct the
            information from
        index: int
            the index of the results file in self.result_files
        """
        f = h5py.File(result_file, "r")
        p = [i for i in f["posterior_samples/%s/%s/parameter_names" % (
            self.labels[index], self.approximant[index])]]
        p = [i.decode("utf-8") for i in p]
        s = [i for i in f["posterior_samples/%s/%s/samples" % (
            self.labels[index], self.approximant[index])]]
        inj_p = [i for i in f["posterior_samples/%s/%s/injection_parameters" % (
            self.labels[index], self.approximant[index])]]
        inj_p = [i.decode("utf-8") for i in inj_p]
        inj_value = [i for i in f["posterior_samples/%s/%s/injection_data" % (
            self.labels[index], self.approximant[index])]]
        injection = {i: j for i, j in zip(inj_p, inj_value)}
        return p, s, injection

    def _data_from_existing_file(self, result_file, index):
        """Extract data from a file in the existing format

        Parameters
        ----------
        result_file: str
            path to the results file that you would like to extrct the
            information from
        index: int
            the index of the results file in self.result_files
        """
        p = self.existing_parameters
        s = self.existing_samples
        injection = [
            {i: float("nan") for i in j} for j in self.existing_parameters]
        return p, s, injection

    @property
    def injection_data(self):
        return self._injection_data

    @property
    def samples(self):
        return self._samples

    @property
    def maxL_samples(self):
        return self._maxL_samples

    @maxL_samples.setter
    def maxL_samples(self, maxL_samples):
        key_data = self._key_data()
        maxL_list = []
        for num, i in enumerate(self.parameters):
            dictionary = {j: key_data[num][j]["maxL"] for j in i}
            dictionary["approximant"] = self.approximant[num]
            maxL_list.append(dictionary)
        self._maxL_samples = maxL_list

    @property
    def same_parameters(self):
        return self._same_parameters

    @same_parameters.setter
    def same_parameters(self, same_parameters):
        params = list(set.intersection(*[set(l) for l in self.parameters]))
        self._same_parameters = params

    @property
    def label_to_prepend_approximant(self):
        labels = [i[len(self.gracedb) + 1:] if self.gracedb else i for i in
                  self.labels]
        prepend = [None] * len(self.approximant)
        duplicates = dict(set(
            (x, self.approximant.count(x)) for x in filter(
                lambda rec: self.approximant.count(rec) > 1,
                self.approximant)))
        if len(duplicates.keys()) > 0:
            for num, i in enumerate(self.approximant):
                if i in duplicates.keys():
                    prepend[num] = labels[num]
        return prepend

    @property
    def psd_labels(self):
        return [self._IFO_from_file_name(i) for i in self.psds]

    @staticmethod
    def _IFO_from_file_name(file):
        """Return a guess of the IFO from the file name.

        Parameters
        ----------
        file: str
            the name of the file that you would like to make a guess for
        """
        file_name = file.split("/")[-1]
        if any(j in file_name for j in ["H", "_0", "IFO0"]):
            ifo = "H1"
        elif any(j in file_name for j in ["L", "_1", "IFO1"]):
            ifo = "L1"
        elif any(j in file_name for j in ["V", "_2", "IFO2"]):
            ifo = "V1"
        else:
            ifo = file_name
        return ifo

    def _key_data(self):
        """Grab the mean, median, maximum likelihood value and the standard
        deviation of each posteiror distribution for each results file.
        """
        key_data_list = []
        for num, i in enumerate(self.samples):
            data = {}
            likelihood_ind = self.parameters[num].index("log_likelihood")
            logL = [j[likelihood_ind] for j in i]
            for ind, j in enumerate(self.parameters[num]):
                index = self.parameters[num].index("%s" % (j))
                subset = [k[index] for k in i]
                data[j] = {"mean": np.mean(subset),
                           "median": np.median(subset),
                           "maxL": subset[logL.index(np.max(logL))],
                           "std": np.std(subset)}
            key_data_list.append(data)
        return key_data_list

    def _grab_frequencies_from_psd_data_file(self, file):
        """Return the frequencies stored in the psd data files

        Parameters
        ----------
        file: str
            path to the psd data file
        """
        fil = open(file)
        fil = fil.readlines()
        fil = [i.strip().split() for i in fil]
        return [float(i[0]) for i in fil]

    def _grab_strains_from_psd_data_file(sef, file):
        """Return the strains stored in the psd data files

        Parameters
        ----------
        file: str
            path to the psd data file
        """
        fil = open(file)
        fil = fil.readlines()
        fil = [i.strip().split() for i in fil]
        return [float(i[1]) for i in fil]



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from distutils.core import setup
from setuptools import find_packages

import subprocess

version = "0.1.5"


def full_description():
    """Get the full readme
    """
    with open("README.md", "r") as f:
        readme = f.read()
    return readme


def write_version_file(version):
    """Add the version number and the git hash to the file
    'pesummary.__init__.py'

    Parameters
    ----------
    version: str
        the release version of the code that you are running
    """
    git_log = subprocess.check_output(
        ["git", "log", "-1", "--pretty=format:%h"]).decode("utf-8")
    with open("pesummary/.version", "w") as f:
        f.writelines(["%s %s" % (version, git_log)])
    return ".version"


readme = full_description()
version_file = write_version_file(version)

setup(name='pesummary',
      version=version,
      description='Python package to produce summary pages for Parameter '
                  'estimation codes',
      author='Charlie Hoy',
      author_email='charlie.hoy@ligo.org',
      url='https://git.ligo.org/lscsoft/pesummary',
      download_url='https://git.ligo.org/lscsoft/pesummary',
      install_requires=[
          'h5py',
          'numpy',
          'corner',
          'matplotlib',
          'deepdish',
          'pandas',
          'pygments',
          'astropy',
          'lalsuite',
          'pytest'],
      include_package_data=True,
      packages=find_packages(),
      package_dir={'pesummary': 'pesummary'},
      package_data={'pesummary': ['js/*.js', 'css/*.css', version_file]},
      entry_points={
          'console_scripts': [
              'summaryconvert=pesummary.file.one_format:main',
              'summarypages=pesummary.summarypages:main',
              'summaryplots=pesummary.summaryplots:main']},
      classifiers=[
          "Programming Language :: Python :: 3.5",
          "Programming Language :: Python :: 3.6"],
      license='MIT',
      long_description=readme,
      long_description_content_type='text/markdown')



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import numpy as np

from pesummary.utils.utils import logger

try:
    from lalsimulation import SimInspiralTransformPrecessingNewInitialConditions
    from lalsimulation import SimInspiralTransformPrecessingWvf2PE
    from lal import MSUN_SI
    LALINFERENCE_INSTALL = True
except ImportError:
    LALINFERENCE_INSTALL = False

try:
    from astropy.cosmology import z_at_value, Planck15
    import astropy.units as u
    ASTROPY = True
except ImportError:
    ASTROPY = False
    logger.warning("You do not have astropy installed currently. You will"
                   " not be able to use some of the prebuilt functions.")


@np.vectorize
def z_from_dL(luminosity_distance):
    """Return the redshift given samples for the luminosity distance
    """
    return z_at_value(Planck15.luminosity_distance, luminosity_distance * u.Mpc)


def dL_from_z(redshift):
    """Return the luminosity distance given samples for the redshift
    """
    return Planck15.luminosity_distance(redshift).value


def comoving_distance_from_z(redshift):
    """Return the comoving distance given samples for the redshift
    """
    return Planck15.comoving_distance(redshift).value


def m1_source_from_m1_z(mass1, z):
    """Return the source mass of the bigger black hole given samples for the
    detector mass of the bigger black hole and the redshift
    """
    return mass1 / (1. + z)


def m2_source_from_m2_z(mass2, z):
    """Return the source mass of the smaller black hole given samples for the
    detector mass of the smaller black hole and the redshift
    """
    return mass2 / (1. + z)


def m_total_source_from_mtotal_z(total_mass, z):
    """Return the source total mass of the binary given samples for detector
    total mass and redshift
    """
    return total_mass / (1. + z)


def mtotal_from_mtotal_source_z(total_mass_source, z):
    """Return the total mass of the binary given samples for the source total
    mass and redshift
    """
    return total_mass_source * (1. + z)


def mchirp_source_from_mchirp_z(mchirp, z):
    """Return the source chirp mass of the binary given samples for detector
    chirp mass and redshift
    """
    return mchirp / (1. + z)


def mchirp_from_mchirp_source_z(mchirp_source, z):
    """Return the chirp mass of the binary given samples for the source chirp
    mass and redshift
    """
    return mchirp_source * (1. + z)


def mchirp_from_m1_m2(mass1, mass2):
    """Return the chirp mass given the samples for mass1 and mass2

    Parameters
    ----------
    """
    return (mass1 * mass2)**0.6 / (mass1 + mass2)**0.2


def m_total_from_m1_m2(mass1, mass2):
    """Return the total mass given the samples for mass1 and mass2
    """
    return mass1 + mass2


def m1_from_mchirp_q(mchirp, q):
    """Return the mass of the larger black hole given the chirp mass and
    mass ratio
    """
    return (q**(2. / 5.)) * ((1.0 + q)**(1. / 5.)) * mchirp


def m2_from_mchirp_q(mchirp, q):
    """Return the mass of the smaller black hole given the chirp mass and
    mass ratio
    """
    return (q**(-3. / 5.)) * ((1.0 + q)**(1. / 5.)) * mchirp


def eta_from_m1_m2(mass1, mass2):
    """Return the symmetric mass ratio given the samples for mass1 and mass2
    """
    return (mass1 * mass2) / (mass1 + mass2)**2


def q_from_m1_m2(mass1, mass2):
    """Return the mass ratio given the samples for mass1 and mass2
    """
    return mass1 / mass2


def q_from_eta(symmetric_mass_ratio):
    """Return the mass ratio given samples for symmetric mass ratio
    """
    temp = (1 / symmetric_mass_ratio / 2 - 1)
    return temp - (temp ** 2 - 1) ** 0.5


def mchirp_from_mtotal_q(total_mass, mass_ratio):
    """Return the chirp mass given samples for total mass and mass ratio
    """
    mass1 = mass_ratio * total_mass / (1. + mass_ratio)
    mass2 = total_mass / (1. + mass_ratio)
    return eta_from_m1_m2(mass1, mass2)**(3. / 5) * (mass1 + mass2)


def chi_p(mass1, mass2, spin1x, spin1y, spin2x, spin2y):
    """Return chi_p given samples for mass1, mass2, spin1x, spin1y, spin2x,
    spin2y
    """
    mass_ratio = mass1 / mass2
    B1 = 2.0 + 1.5 * mass_ratio
    B2 = 2.0 + 3.0 / (2 * mass_ratio)
    S1_perp = ((spin1x)**2 + (spin1y)**2)**0.5
    S2_perp = ((spin2x)**2 + (spin2y)**2)**0.5
    chi_p = 1.0 / B1 * np.maximum(B1 * S1_perp, B2 * S2_perp)
    return chi_p


def chi_eff(mass1, mass2, spin1z, spin2z):
    """Return chi_eff given samples for mass1, mass2, spin1z, spin2z
    """
    return (spin1z * mass1 + spin2z * mass2) / (mass1 + mass2)


def phi_12_from_phi1_phi2(phi1, phi2):
    """Return the difference in azimuthal angle between S1 and S2 given samples
    for phi1 and phi2
    """
    phi12 = phi2 - phi1
    if isinstance(phi12, float) and phi12 < 0.:
        phi12 += 2 * np.pi
    elif isinstance(phi12, np.ndarray):
        ind = np.where(phi12 < 0.)
        phi12[ind] += 2 * np.pi
    return phi12


def spin_angles(mass_1, mass_2, inc, spin1x, spin1y, spin1z, spin2x, spin2y,
                spin2z, f_ref, phase):
    """Return the spin angles given samples for mass_1, mass_2, inc, spin1x,
    spin1y, spin1z, spin2x, spin2y, spin2z, f_ref, phase
    """
    if LALINFERENCE_INSTALL:
        data = []
        for i in range(len(mass_1)):
            theta_jn, phi_jl, tilt_1, tilt_2, phi_12, a_1, a_2 = \
                SimInspiralTransformPrecessingWvf2PE(
                    incl=inc[i], m1=mass_1[i], m2=mass_2[i], S1x=spin1x[i],
                    S1y=spin1y[i], S1z=spin1z[i], S2x=spin2z[i], S2y=spin2y[i],
                    S2z=spin2z[i], fRef=float(f_ref[i]), phiRef=phase[i])
            data.append([theta_jn, phi_jl, tilt_1, tilt_2, phi_12, a_1, a_2])
        return data


def component_spins(theta_jn, phi_jl, tilt_1, tilt_2, phi_12, a_1, a_2, mass_1,
                    mass_2, f_ref, phase):
    """Return the component spins given samples for theta_jn, phi_jl, tilt_1,
    tilt_2, phi_12, a_1, a_2, mass_1, mass_2, f_ref, phase
    """
    if LALINFERENCE_INSTALL:
        data = []
        for i in range(len(theta_jn)):
            iota, S1x, S1y, S1z, S2x, S2y, S2z = \
                SimInspiralTransformPrecessingNewInitialConditions(
                    theta_jn[i], phi_jl[i], tilt_1[i], tilt_2[i], phi_12[i],
                    a_1[i], a_2[i], mass_1[i] * MSUN_SI, mass_2[i] * MSUN_SI,
                    float(f_ref[i]), phase[i])
            data.append([iota, S1x, S1y, S1z, S2x, S2y, S2z])
        return data
    else:
        raise Exception("Please install LALSuite for full conversions")



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from glob import glob

import h5py
import json


class ExistingFile():
    """This class handles the existing posterior_samples.h5 file

    Parameters
    ----------
    parser: argparser
        The parser containing the command line arguments

    Attributes
    ----------
    existing_file: str
        the path to the existing posterior_samples.h5 file
    existing_approximants: list
        list of approximants that have been used in the previous analysis
    existing_labels: list
        list of labels that have been used in the previous analysis
    existing_samples: nd list
        nd list of samples stored for each approximant used in the previous
        analysis
    """
    def __init__(self, existing_webdir):
        self.existing = existing_webdir
        self.existing_data = []

    @property
    def existing_file(self):
        meta_file = glob(self.existing + "/samples/posterior_samples*")
        return meta_file[0]

    @property
    def extension(self):
        return self.existing_file.split(".")[-1]

    @property
    def existing_data(self):
        return self._existing_data

    @existing_data.setter
    def existing_data(self, existing_data):
        func_map = {"h5": self._grab_data_from_hdf5_file,
                    "json": self._grab_data_from_json_file}
        self._existing_data = func_map[self.extension]()

    def _grab_data_from_hdf5_file(self):
        """
        """
        f = h5py.File(self.existing_file)
        existing_data = self._grab_data_from_dictionary(f)
        f.close()
        return existing_data

    def _grab_data_from_json_file(self):
        with open(self.existing_file) as f:
            data = json.load(f)
        return self._grab_data_from_dictionary(data)

    def _grab_data_from_dictionary(self, dictionary):
        """
        """
        labels = list(dictionary["posterior_samples"].keys())
        existing_structure = {
            i: j for i in labels for j in
            dictionary["posterior_samples"]["%s" % (i)].keys()}
        labels = list(existing_structure.keys())
        approximants = [existing_structure[i] for i in labels]

        parameter_list, sample_list = [], []
        for num, i in enumerate(labels):
            p = [j for j in dictionary["posterior_samples"]["%s" % (i)]["%s" % (
                approximants[num])]["parameter_names"]]
            s = [j for j in dictionary["posterior_samples"]["%s" % (i)]["%s" % (
                 approximants[num])]["samples"]]
            if isinstance(p[0], bytes):
                parameter_list.append([j.decode("utf-8") for j in p])
            else:
                parameter_list.append([j for j in p])
            sample_list.append(s)
        return labels, approximants, parameter_list, sample_list

    @property
    def existing_labels(self):
        return self.existing_data[0]

    @property
    def existing_approximant(self):
        return self.existing_data[1]

    @property
    def existing_parameters(self):
        return self.existing_data[2]

    @property
    def existing_samples(self):
        return self.existing_data[3]
